// CNY.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "CNY.h"


void VectorLength( float x, float y, float z , float& out)
{
	out = (float)sqrt(x * x + y * y + z * z);
}

//-------------------------------------------------------------------
void VectorNormalize( float x, float y, float z, float& outx, float& outy, float& outz )
{
	float l;
	VectorLength( x, y, z , l);

	outx = x/l;
	outy = y/l;
	outz = z/l;
}

//-------------------------------------------------------------------
void VectorMultiply( float v1x, float v1y, float v1z, float v2x, float v2y, float v2z,
					float& outx, float& outy, float& outz)
{
	outx = v1y * v2z - v1z * v2y;
	outy = v1z * v2x - v1x * v2z;
	outz = v1x * v2y - v1y * v2x;
}

//-------------------------------------------------------------------
void VectorScalarMultiply( float vx, float vy, float vz, float s, float& outx, float& outy, float& outz )
{
	outx = vx * s;
	outy = vy * s;
	outz = vz * s;
}

//-------------------------------------------------------------------
void VectorDiff( float v1x, float v1y, float v1z, float v2x, float v2y, float v2z , float& outx, float& outy, float& outz)
{
	outx = v1x - v2x;
	outy = v1y - v2y;
	outz = v1z - v2z;
}

typedef struct
{
	float x;
	float y;
	float z;
} Vector;


void DrawParticleLuce(float x, float y, float z, float size, float vx, float vy, float vz)
{
	Vector Position;
	Vector MyPosition;
	Position.x = x;
	Position.y = y;
	Position.z = z;

	MyPosition.x=vx;
	MyPosition.y=vy;
	MyPosition.z=vz;

	Vector sight;
	VectorDiff(MyPosition.x, MyPosition.y, MyPosition.z, Position.x, Position.y, Position.z,
		sight.x, sight.y, sight.z);

	Vector cz;
	cz.x = 0.0f;
	cz.y = 0.0f;
	cz.z = 1.0f;

	Vector cross1;
	Vector cross2;
	VectorMultiply( sight.x, sight.y, sight.z, cz.x, cz.y, cz.z,
		cross1.x, cross1.y, cross1.z);

	VectorMultiply( sight.x, sight.y, sight.z , cross1.x, cross1.y, cross1.z,
		cross2.x, cross2.y, cross2.z);

	VectorNormalize(cross1.x, cross1.y, cross1.z, cross1.x, cross1.y, cross1.z);
	VectorNormalize(cross2.x, cross2.y, cross2.z, cross2.x, cross2.y, cross2.z);

	VectorScalarMultiply(cross1.x, cross1.y, cross1.z, size, cross1.x, cross1.y, cross1.z);
	VectorScalarMultiply(cross2.x, cross2.y, cross2.z, size, cross2.x, cross2.y, cross2.z);

	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f( 1.0f, 1.0f );
		glVertex3f( Position.x - cross1.x, Position.y - cross1.y, Position.z - cross1.z);
		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f( Position.x + cross2.x, Position.y + cross2.y, Position.z + cross2.z);
		glTexCoord2f( 1.0f, 0.0f );
		glVertex3f( Position.x - cross2.x, Position.y - cross2.y, Position.z - cross2.z);
		glTexCoord2f( 0.0f, 0.0f );
		glVertex3f( Position.x + cross1.x, Position.y + cross1.y, Position.z + cross1.z);
	glEnd();
}


CNYApp theApp;

CNYApp* GetThisApp()
{
	return &theApp;
}


void CNYApp::AddRocket()
{
	Pointer<Rocket> pRocket = new Rocket;
	pRocket->m_bExploded = false;
	pRocket->m_bSetOff = false;
	pRocket->lighterPos = 180.0f;
	pRocket->x = (((float)(rand()%1000)/1000.0f)*5.0f)-2.5f;
	pRocket->m_fStartDelay = (((float)(rand()%1000)/1000.0f)*3.0f);
	pRocket->y = 0;
	pRocket->z = 2.0;
	pRocket->angle = 20.0f;
	pRocket->m_fShootDelay = (((float)(rand()%1000)/1000.0f)*2.0f)+1.0f;
	pRocket->m_fShootSpeed = (((float)(rand()%1000)/1000.0f)*5.0f)+10.0f;
	pRocket->blowAngle = (((float)(rand()%1000)/1000.0f)*20.0f)+50.0f;

	int clr = rand()%6;

	switch (clr)
	{
	case 0:
		pRocket->exploder = 1.0f;
		pRocket->explodeg = 0.0f;
		pRocket->explodeb = 0.0f;
		break;
	case 1:
		pRocket->exploder = 0.0f;
		pRocket->explodeg = 1.0f;
		pRocket->explodeb = 0.0f;
		break;
	case 2:
		pRocket->exploder = 0.5f;
		pRocket->explodeg = 0.5f;
		pRocket->explodeb = 1.0f;
		break;
	case 3:
		pRocket->exploder = 1.0f;
		pRocket->explodeg = 1.0f;
		pRocket->explodeb = 0.0f;
		break;
	case 4:
		pRocket->exploder = 1.0f;
		pRocket->explodeg = 0.5f;
		pRocket->explodeb = 1.0f;
		break;
	case 5:
		pRocket->exploder = 0.0f;
		pRocket->explodeg = 1.0f;
		pRocket->explodeb = 1.0f;
		break;
	}

	m_aRockets.Insert(pRocket);
}

#define EMPTY_SOUND		(-128)
#define WAVE_BUFFERS	(3)

void CNYApp::MixSounds(LPWAVEHDR waveHdr)
{
	memset((unsigned char*)waveHdr->lpData, EMPTY_SOUND, waveHdr->dwBufferLength);
	int cnt;
	for (cnt = 0; cnt < m_aSoundInstances.GetSize(); cnt++)
	{
		if (!m_aSoundInstances[cnt]->MixData(waveHdr))
		{
			m_aSoundInstances.RemoveAt(cnt);
			cnt--;
		}
	}
}

void CALLBACK MywaveOutProc(
  HWAVEOUT hwo,      
  UINT uMsg,         
  DWORD dwInstance,  
  DWORD dwParam1,    
  DWORD dwParam2     
)
{
	LPWAVEHDR pWaveHdr;
	switch(uMsg)
	{
	case MM_WOM_OPEN:
		break;
	case MM_WOM_DONE:
		pWaveHdr = (LPWAVEHDR)dwParam1;
		if (GetThisApp()->m_bCloseSoundsFlag)
		{
			if (waveOutUnprepareHeader(GetThisApp()->hWaveOut, pWaveHdr, sizeof(WAVEHDR)) != MMSYSERR_NOERROR)
			{
				GFC::GetDebugger()->ErrorOut("waveOutUnprepareHeader failed!\n");
			}
			free(pWaveHdr->lpData);
			delete pWaveHdr;
			GetThisApp()->m_nCloseSoundCount++;
			if (GetThisApp()->m_nCloseSoundCount == WAVE_BUFFERS)
			{
				waveOutClose(GetThisApp()->hWaveOut);
			}
			GetThisApp()->hWaveOut = NULL;
		} else
		{
			GetThisApp()->MixSounds(pWaveHdr);
			if (waveOutWrite(GetThisApp()->hWaveOut, pWaveHdr, sizeof(WAVEHDR)) != MMSYSERR_NOERROR)
			{
				GFC::GetDebugger()->ErrorOut("waveOutWrite in callback failed!\n");
			}
		}

		break;
	case MM_WOM_CLOSE:
		break;
	}
}


void CNYApp::InitSound()
{
	WAVEFORMATEX format;
	format.cbSize = sizeof(WAVEFORMATEX);
	format.wFormatTag = WAVE_FORMAT_PCM;
	format.nChannels = 1;
	format.wBitsPerSample = 8;
	format.nSamplesPerSec = 11025;
	format.nBlockAlign = 2;
	format.nAvgBytesPerSec = 2*11025;
	m_bCloseSoundsFlag = false;
	if (waveOutOpen(&hWaveOut, WAVE_MAPPER, &format,
				(DWORD)MywaveOutProc, 0, CALLBACK_FUNCTION) != MMSYSERR_NOERROR)
	{
		GFC::GetDebugger()->ErrorOut("*** waveOutOpen failed!\n");
	} else
	{
		for (int i = 0; i < WAVE_BUFFERS; i++)
		{
			LPWAVEHDR waveHdr = new WAVEHDR;
			waveHdr->dwBufferLength = format.nAvgBytesPerSec/25;
			waveHdr->lpData = (char*)malloc(waveHdr->dwBufferLength);
			memset((unsigned char*)waveHdr->lpData, EMPTY_SOUND, waveHdr->dwBufferLength);
			waveHdr->dwFlags = 0;
			waveHdr->dwLoops = 0;
			if (waveOutPrepareHeader(hWaveOut, waveHdr, sizeof(WAVEHDR)) == MMSYSERR_NOERROR)
			{
				if (waveOutWrite(hWaveOut, waveHdr, sizeof(WAVEHDR)) != MMSYSERR_NOERROR)
				{
					GFC::GetDebugger()->ErrorOut("*** waveOutWrite failed!\n");
					waveOutUnprepareHeader(hWaveOut, waveHdr, sizeof(WAVEHDR));
					delete waveHdr;
					waveOutClose(hWaveOut);
					hWaveOut = NULL;
				}
			} else
			{
				GFC::GetDebugger()->ErrorOut("*** waveOutPrepareHeader failed!\n");
			}
		}
	}
}

void CNYApp::CloseSounds()
{
	m_bCloseSoundsFlag = true;
	m_nCloseSoundCount = 0;
}

#define SCREEN_SAVER

// checks a screensaver's command line and
// returns true if the program should continue
bool SSCmdLine_ShouldContinue(const String& lpCmdLine)
{
	return (stricmp(lpCmdLine, "/s") == 0 && FindWindow(NULL, "CNY Application") == NULL);
}


bool CNYApp::Init()
{
#ifdef SCREEN_SAVER
	if (SSCmdLine_ShouldContinue(GFC::GetCommandLine()) == false)
		return false; // stop running
#endif

	gldFov = 60.0;
	rx = -17.0f;
	ry = 20.0f;
	px = -3.0f;
	py = -0.8f;
	pz = 7.0f;
	m_nPrevMouseX = -200;
	m_nPrevMouseY = -200;

	m_fFlashAlpha = 0.0f;

	AddRocket();
	AddRocket();
	AddRocket();


	textureParticle = 0;
	textureWall = 0;
	textureHelp = 0;

	textureSky = textureBottom = textureLeft = textureRight = textureFront = textureBack = 0;

	GFC::LoadLibrary("SBObject.dll");

	GFC::GetInput()->SetInputTarget((InputTarget*)&theApp);
	GFC::GetWindow()->SetOpenGL((OGL*)&theApp);

	GFC::GetDebugger()->DebugOut("Creating window...\n");
	if (GFC::GetWindow()->CreateWindow("CNY Application"))
	{
		InitSound();

		GFC::GetWindow()->ShowWindow();
		GFC::GetWindow()->SetFullScreen();
		ShowCursor(FALSE);

		GFC::GetDebugger()->DebugOut("Window is displayed.\n");
		return true;
	} else
	{
		GFC::GetDebugger()->ErrorOut("Unable to create window\n");
		return false;
	}
}

void CNYApp::OnIdle()
{
	double rad;
	float t_fTimePassed = GetTimePassed();
	float t_ry;

	if (m_fFlashAlpha > 0.0f)
	{
		m_fFlashAlpha -= GetTimePassed();
		if (m_fFlashAlpha < 0.0f)
			m_fFlashAlpha = 0.0f;
	}

	int cnt;
	for (cnt = 0; cnt < m_aRockets.GetSize(); cnt++)
	{
		m_aRockets[cnt]->Update(GetTimePassed());
	}
	

	float SPEED;
	// check shift key
	if (GFC::GetInput()->IsKeyDown(0x10))
		SPEED = 10.0f;
	else
		SPEED = 5.0f;

	GLfloat cy, sy;
	GLfloat cy2, sy2;
	if (GFC::GetInput()->IsKeyDown('W')
		|| GFC::GetInput()->IsKeyDown(VK_UP)
		|| GFC::GetInput()->IsKeyDown(VK_NUMPAD8))
	{
		// move forward
		rad = (double)DTOR(ry);
		cy = (GLfloat)cos(rad);
		sy = (GLfloat)sin(rad);

		rad = (double)DTOR(rx);
		cy2 = (GLfloat)cos(rad);
		sy2 = (GLfloat)sin(rad);

		pz -= SPEED*t_fTimePassed * cy * cy2;
		px += SPEED*t_fTimePassed * sy * cy2;
		py -= SPEED*t_fTimePassed * sy2;
	}
	if (GFC::GetInput()->IsKeyDown('A')
		|| GFC::GetInput()->IsKeyDown(VK_LEFT)
		|| GFC::GetInput()->IsKeyDown(VK_NUMPAD4))
	{
		// strafe left
		t_ry = ry-90.0f;
		if (t_ry < 0.0f)
			t_ry += 360.0f;
		rad = (double)DTOR(t_ry);
		cy = (GLfloat)cos(rad);
		sy = (GLfloat)sin(rad);

		pz -= SPEED*t_fTimePassed * cy;
		px += SPEED*t_fTimePassed * sy;
	}
	if (GFC::GetInput()->IsKeyDown('S')
		|| GFC::GetInput()->IsKeyDown(VK_DOWN)
		|| GFC::GetInput()->IsKeyDown(VK_NUMPAD2))
	{
		// move back
		rad = (double)DTOR(ry);
		cy = (GLfloat)cos(rad);
		sy = (GLfloat)sin(rad);

		rad = (double)DTOR(rx);
		cy2 = (GLfloat)cos(rad);
		sy2 = (GLfloat)sin(rad);

		pz += SPEED*t_fTimePassed * cy * cy2;
		px -= SPEED*t_fTimePassed * sy * cy2;
		py += SPEED*t_fTimePassed * sy2;
	}
	if (GFC::GetInput()->IsKeyDown('D')
		|| GFC::GetInput()->IsKeyDown(VK_RIGHT)
		|| GFC::GetInput()->IsKeyDown(VK_NUMPAD6))
	{
		// strafe right
		t_ry = ry-90.0f;
		if (t_ry < 0.0f)
			t_ry += 360.0f;
		rad = (double)DTOR(t_ry);
		cy = (GLfloat)cos(rad);
		sy = (GLfloat)sin(rad);

		pz += SPEED*t_fTimePassed * cy;
		px -= SPEED*t_fTimePassed * sy;
	}
	if (GFC::GetInput()->IsRButtonDown()
		|| GFC::GetInput()->IsKeyDown(VK_PRIOR))
		py += SPEED*t_fTimePassed;
	if (GFC::GetInput()->IsLButtonDown()
		|| GFC::GetInput()->IsKeyDown(VK_NEXT))
		py -= SPEED*t_fTimePassed;
}

void CNYApp::OnExit()
{
	m_aSoundInstances.SetSize(0);
	CloseSounds();

	m_aRockets.SetSize(0);
	m_pCracker = NULL;

	GFC::UnloadLibrary("SBObject.dll");
}


//////////////////////////////////////////////////////////////////////
// methods from OGL interface

#include "TextureLoader.h"
#include "WaveFile.h"

void CNYApp::LoadFiles()
{
	Pointer<ResourceFile> pResource = new ResourceFile;
	pResource->SetModule(GetModuleHandle(NULL));
	Pointer<MultiFile> pMulti = new MultiFile;
	if (pMulti->OpenMultiFile((SBFile*)pResource, "IDR_DATA\nDAT"))
	{
		m_pCracker = (SB3DObject*)RTTIClass::FindClass("SBObject")->CreateObject("SB3DObject");
		m_pCracker->LoadFromFile((SBFile*)pMulti, "cracker.cob");
		m_pCracker->SmoothSurfaces(89.0f);

		textureWall = TextureLoader::LoadTGA((SBFile*)pMulti, "wall.tga");
		textureParticle = TextureLoader::LoadTGA((SBFile*)pMulti, "particle.tga");
		textureHelp = TextureLoader::LoadTGA((SBFile*)pMulti, "help.tga");

		wavLaunch.LoadFile((SBFile*)pMulti, "launch.wav");
		wavExplode.LoadFile((SBFile*)pMulti, "explode.wav");

		textureSky = TextureLoader::LoadTGA((SBFile*)pMulti, "sky.tga");
		textureBottom = TextureLoader::LoadTGA((SBFile*)pMulti, "floor.tga");
		textureRight = TextureLoader::LoadTGA((SBFile*)pMulti, "right.tga");
		textureLeft = TextureLoader::LoadTGA((SBFile*)pMulti, "left.tga");
		textureFront = TextureLoader::LoadTGA((SBFile*)pMulti, "front.tga");
		textureBack = TextureLoader::LoadTGA((SBFile*)pMulti, "back.tga");
	}
}


// callback function after creating rendering context
void CNYApp::OnInitialiseRC()
{
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

    GLfloat glfLightAmbient[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat glfLightDiffuse[] = { 0.9f, 0.3f, 0.3f, 1.0f };
    GLfloat glfLightSpecular[] = { 0.9f, 0.9f, 0.9f, 1.0f };

    //
    // Add a light to the scene.
    //
    glLightfv (GL_LIGHT0, GL_AMBIENT, glfLightAmbient);
    glLightfv (GL_LIGHT0, GL_DIFFUSE, glfLightDiffuse);
    glLightfv (GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
    glEnable (GL_LIGHT0);

    GLfloat glfLightAmbient1[] = { 0.1f, 0.1f, 0.1f, 1.0f };
    GLfloat glfLightDiffuse1[] = { 0.9f, 0.9f, 0.9f, 1.0f };
    GLfloat glfLightSpecular1[] = { 0.9f, 0.9f, 0.9f, 1.0f };

    //
    // Add a light to the scene.
    //
    glLightfv (GL_LIGHT1, GL_AMBIENT, glfLightAmbient1);
    glLightfv (GL_LIGHT1, GL_DIFFUSE, glfLightDiffuse1);
    glLightfv (GL_LIGHT1, GL_SPECULAR, glfLightSpecular1);
    glEnable (GL_LIGHT1);


    glEnable (GL_LIGHTING);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

	glEnable(GL_POLYGON_SMOOTH);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);

	LoadFiles();
}

// callback function before deleting rendering context
void CNYApp::OnDeleteRC()
{
	if (textureParticle != 0)
		glDeleteTextures(1, &textureParticle);
	if (textureWall != 0)
		glDeleteTextures(1, &textureWall);
	if (textureHelp != 0)
		glDeleteTextures(1, &textureHelp);

	if (textureSky != 0)
		glDeleteTextures(1, &textureSky);
	if (textureBottom != 0)
		glDeleteTextures(1, &textureBottom);
	if (textureLeft != 0)
		glDeleteTextures(1, &textureLeft);
	if (textureRight != 0)
		glDeleteTextures(1, &textureRight);
	if (textureFront != 0)
		glDeleteTextures(1, &textureFront);
	if (textureBack != 0)
		glDeleteTextures(1, &textureBack);

	textureParticle = 0;
	textureWall = 0;
	textureHelp = 0;

	textureSky = textureBottom = textureLeft = textureRight = textureFront = textureBack = 0;
}

void DrawBox(float left, float right, float top, float bottom, float in, float out, float texturemult = 1.0f)
{
	float width = (right-left)*texturemult;
	float height = (top-bottom)*texturemult;
	float depth = (out-in)*texturemult;

	// front
	glBegin(GL_TRIANGLE_STRIP);
		glNormal3f(0.0f, 0.0f, -1.0f);
		glTexCoord2f(width, height);
		glVertex3f(right, top, out);
		glTexCoord2f(0.0f, height);
		glVertex3f(left, top, out);
		glTexCoord2f(width, 0.0f);
		glVertex3f(right, bottom, out);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(left, bottom, out);
	glEnd();

	// top
	glBegin(GL_TRIANGLE_STRIP);
		glNormal3f(0.0f, 1.0f, 0.0f);
		glTexCoord2f(width, depth);
		glVertex3f(right, top, in);
		glTexCoord2f(0.0f, depth);
		glVertex3f(left, top, in);
		glTexCoord2f(width, 0.0f);
		glVertex3f(right, top, out);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(left, top, out);
	glEnd();


	// back
	glBegin(GL_TRIANGLE_STRIP);
		glNormal3f(0.0f, 0.0f, 1.0f);
		glTexCoord2f(width, height);
		glVertex3f(left, top, in);
		glTexCoord2f(0.0f, height);
		glVertex3f(right, top, in);
		glTexCoord2f(width, 0.0f);
		glVertex3f(left, bottom, in);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(right, bottom, in);
	glEnd();

	// bottom
	glBegin(GL_TRIANGLE_STRIP);
		glNormal3f(0.0f, -1.0f, 0.0f);
		glTexCoord2f(width, depth);
		glVertex3f(left, bottom, in);
		glTexCoord2f(0.0f, depth);
		glVertex3f(right, bottom, in);
		glTexCoord2f(width, 0.0f);
		glVertex3f(left, bottom, out);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(right, bottom, out);
	glEnd();


	// right
	glBegin(GL_TRIANGLE_STRIP);
		glNormal3f(1.0f, 0.0f, 0.0f);
		glTexCoord2f(depth, height);
		glVertex3f(right, top, in);
		glTexCoord2f(0.0f, height);
		glVertex3f(right, top, out);
		glTexCoord2f(depth, 0.0f);
		glVertex3f(right, bottom, in);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(right, bottom, out);
	glEnd();

	// left
	glBegin(GL_TRIANGLE_STRIP);
		glNormal3f(-1.0f, 0.0f, 0.0f);
		glTexCoord2f(depth, height);
		glVertex3f(left, top, out);
		glTexCoord2f(0.0f, height);
		glVertex3f(left, top, in);
		glTexCoord2f(depth, 0.0f);
		glVertex3f(left, bottom, out);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(left, bottom, in);
	glEnd();
}

#define SB_PI		3.1415926535897932384626433832795
#define RTOD(a)		(((a)*180.0)/SB_PI)
#define DTOR(a)		(((a)*SB_PI)/180.0)

#define SKYBOX_DISTANCE		(200.0f)
#define SKYBOX_BOTTOM		(199.0f)

// callback function for drawing scene
void CNYApp::OnDrawScene()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	glRotatef(rx, 1.0f, 0.0f, 0.0f);
	glRotatef(ry, 0.0f, 1.0f, 0.0f);
	glTranslatef(-px, -py, -pz);

// start drawing skybox
	glDisable(GL_LIGHTING);
	glDisable(GL_BLEND);
	glDepthMask(FALSE);
	glEnable(GL_TEXTURE_2D);
	glColor3f(1.0f, 1.0f, 1.0f);

	// top
	glBindTexture(GL_TEXTURE_2D, textureSky);
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, SKYBOX_DISTANCE, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, SKYBOX_DISTANCE, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, SKYBOX_DISTANCE, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, SKYBOX_DISTANCE, -SKYBOX_DISTANCE-1.0f);
	glEnd();

	// bottom
	glBindTexture(GL_TEXTURE_2D, textureBottom);
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(2.0f, 2.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(0.0f, 2.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(2.0f, 0.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, +SKYBOX_DISTANCE+1.0f);
	glEnd();

	// front
	glBindTexture(GL_TEXTURE_2D, textureFront);
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, SKYBOX_DISTANCE, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, SKYBOX_DISTANCE, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, -SKYBOX_DISTANCE-1.0f);
	glEnd();

	// back
	glBindTexture(GL_TEXTURE_2D, textureBack);
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, SKYBOX_DISTANCE, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, SKYBOX_DISTANCE, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, SKYBOX_DISTANCE+1.0f);
	glEnd();

	// left
	glBindTexture(GL_TEXTURE_2D, textureLeft);
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, SKYBOX_DISTANCE, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, SKYBOX_DISTANCE, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-SKYBOX_DISTANCE-1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, SKYBOX_DISTANCE+1.0f);
	glEnd();

	// right
	glBindTexture(GL_TEXTURE_2D, textureRight);
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, SKYBOX_DISTANCE, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, SKYBOX_DISTANCE, -SKYBOX_DISTANCE-1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, SKYBOX_DISTANCE+1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(SKYBOX_DISTANCE+1.0f, -SKYBOX_DISTANCE+SKYBOX_BOTTOM, -SKYBOX_DISTANCE-1.0f);
	glEnd();

	glDepthMask(TRUE);
	glEnable(GL_LIGHTING);
// finish drawing skybox

	GLfloat lightpos0[] = { -7.0f, 2.0f, -6.0f, 1.0f };
	glLightfv(GL_LIGHT0, GL_POSITION, lightpos0);

	GLfloat lightpos1[] = { 7.0f, 2.0f, -6.0f, 1.0f };
	glLightfv(GL_LIGHT1, GL_POSITION, lightpos1);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureWall);
	glColor3f(0.7f, 0.7f, 0.7f);

	// front block
	DrawBox(-5.0f, 5.0f, 0.0f, -1.0f, -1.0f, 1.0f);
	// left block
	DrawBox(-5.0f, -4.0f, 0.0f, -1.0f, 1.0f, 10.0f);
	// right block
	DrawBox(4.0f, 5.0f, 0.0f, -1.0f, 1.0f, 10.0f);
	// floor
	DrawBox(-4.0f, 4.0f, -0.95f, -1.0f, 1.0f, 10.0f);

	glDisable(GL_TEXTURE_2D);

	int cnt;
	for (cnt = 0; cnt < m_aRockets.GetSize(); cnt++)
	{
		m_aRockets[cnt]->RenderRocket();
	}
	for (cnt = 0; cnt < m_aRockets.GetSize(); cnt++)
	{
		m_aRockets[cnt]->Render(-px, -py, -pz);
	}

	SetOrtho();
	glLoadIdentity();

	glEnable(GL_BLEND);
	glDisable(GL_LIGHTING);
	glDepthMask(FALSE);
	glDisable(GL_DEPTH_TEST);

	// draw flash
	if (m_fFlashAlpha > 0.0f)
	{
		glColor4f(1.0f, 1.0f, 1.0f, m_fFlashAlpha);
		glBegin(GL_TRIANGLE_STRIP);
		glVertex3f((float)nWinCX, (float)nWinCY, 0.0f);
		glVertex3f(0.0f, (float)nWinCY, 0.0f);
		glVertex3f((float)nWinCX, 0.0f, 0.0f);
		glVertex3f(0.0f, 0.0f, 0.0f);
		glEnd();
	}

	glEnable(GL_TEXTURE_2D);




	glColor4f(0.0f, 1.0f, 1.0f, 1.0f);

// draw help message
	glBindTexture(GL_TEXTURE_2D, textureHelp);
	glBegin(GL_TRIANGLE_STRIP);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(180.0f, 40.0f, 0.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(30.0f, 40.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(180.0f, 0.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(30.0f, 0.0f, 0.0f);
	glEnd();

	glEnable(GL_DEPTH_TEST);
	glDepthMask(TRUE);
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glDisable(GL_BLEND);

	SetPerspective();

	glFlush();
	glFinish();
}

// callback function when window size changes
void CNYApp::OnWindowSizeChange(int nWidth, int nHeight)
{
	nWinCX = nWidth;
	nWinCY = nHeight;
	SetPerspective();
}

//////////////////////////////////////////////////////////////////////
// methods from input target

// callback function for keyboard events
void CNYApp::KeyboardFunction(int nEvent, int nKeyCode)
{
	if (nEvent == KB_KEYDOWN)
	{
		if (nKeyCode == 'P')
		{
			GFC::GetDebugger()->DebugOut("px = %f;\npy = %f;\npz = %f;\nrx = %f;\nry = %f;\n",
											px, py, pz, rx, ry);
		} else if (nKeyCode == VK_ADD)
		{
			AddRocket();
		} else if (nKeyCode == VK_SUBTRACT)
		{
			if (m_aRockets.GetSize() > 0)
				m_aRockets.RemoveAt(0);
		}
	}
}

#define MOUSEROT_SPEED	0.5f;
// callback function for mouse events
void CNYApp::MouseFunction(int nEvent, int nPosX, int nPosY)
{
	if (nEvent == MOUSE_MOVE)
	{

		if (m_nPrevMouseX != -200 || m_nPrevMouseY != -200)
		{
			GLfloat dX, dY;
			dX = (nPosY-m_nPrevMouseY)*MOUSEROT_SPEED;
			dY = (nPosX-m_nPrevMouseX)*MOUSEROT_SPEED;
			rx += dX;
			ry += dY;
			while (ry >= 360.0f)
				ry -= 360.0f;
			while (ry < 0.0f)
				ry += 360.0f;
			while (rx > 90.0f)
				rx = 90.0f;
			while (rx < -90.0f)
				rx = -90.0f;
		}
		// move mouse back to center of screen
		m_nPrevMouseX = -1;
		m_nPrevMouseY = -1;
		GFC::GetInput()->SetMousePosition(m_nPrevMouseX, m_nPrevMouseY);
	}
}


inline void CNYApp::SetOrtho()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0.0f, nWinCX, 0.0f, nWinCY, -nWinCX, nWinCX);


	glViewport(0, 0, nWinCX, nWinCY);

	glMatrixMode(GL_MODELVIEW);
}

inline void CNYApp::SetPerspective()
{
	gldAspect = (GLdouble) nWinCX / (GLdouble) nWinCY;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(gldFov,			// Field-of-view angle
					gldAspect,		// Aspect ratio of viewing volume
					0.1,			// Distance to near clipping plane
					1000.0);			// Distance to far clipping plane

	glViewport(0, 0, nWinCX, nWinCY);

	glMatrixMode(GL_MODELVIEW);
}


