// ExplosionSystem.cpp: implementation of the ExplosionSystem class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CNY.h"
#include "Rocket.h"
#include "ExplosionSystem.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ExplosionSystem::ExplosionSystem()
{

}

ExplosionSystem::~ExplosionSystem()
{

}


void ExplosionSystem::InitAll()
{
	m_nDead = 0;
	int cnt;
	for (cnt = 0; cnt < MAX_EXPLOSIONS; cnt++)
	{
		InitElement(cnt);
	}
}

inline void ExplosionSystem::InitElement(int index)
{
	ep[index].x = m_pRocket->x;
	ep[index].y = m_pRocket->y;
	ep[index].z = m_pRocket->z;

	ep[index].alpha = (((float)(rand()%1000)/1000.0f)*0.2f)+0.8f;
	ep[index].cx = (((float)(rand()%1000)/1000.0f)*10.0f)+0.2f;
	ep[index].cz = (((float)(rand()%1000)/1000.0f)*10.0f)+0.2f;
	ep[index].cy = (((float)(rand()%1000)/1000.0f)*10.0f)-1.0f;

	int dir = rand()%4;
	switch (dir)
	{
	case 0:
		ep[index].cx = -ep[index].cx;
		break;
	case 1:
		ep[index].cz = -ep[index].cz;
		break;
	case 2:
		ep[index].cx = -ep[index].cx;
		ep[index].cz = -ep[index].cz;
		break;
	}
}

void ExplosionSystem::Render(float vx, float vy, float vz)
{
	int cnt;
	glEnable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D, GetThisApp()->textureParticle);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_CULL_FACE);
	glDepthMask(FALSE);
	for (cnt = 0; cnt < MAX_EXPLOSIONS; cnt++)
	{
		if (ep[cnt].alpha > 0.0f)
		{
			glColor4f(m_pRocket->exploder, m_pRocket->explodeg, m_pRocket->explodeb, ep[cnt].alpha);
			DrawParticleLuce(ep[cnt].x, ep[cnt].y, ep[cnt].z, 0.25f, vx, vy, vz);
		}
	}
	glDepthMask(TRUE);
	glEnable(GL_CULL_FACE);
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
}

#define GRAVITY		(-10.0f)

void ExplosionSystem::Update(float fTimePassed)
{
	int cnt;
	for (cnt = 0; cnt < MAX_EXPLOSIONS; cnt++)
	{
		if (ep[cnt].alpha > 0.0f)
		{
			ep[cnt].alpha -= fTimePassed*0.3f;
			if (ep[cnt].alpha <= 0.0f)
			{
				m_nDead++;
			} else
			{
				ep[cnt].x += fTimePassed*ep[cnt].cx;
				ep[cnt].y += fTimePassed*ep[cnt].cy;
				ep[cnt].z += fTimePassed*ep[cnt].cz;
				ep[cnt].cy += fTimePassed*GRAVITY;
			}
		}
	}
}


