// Rocket.cpp: implementation of the Rocket class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CNY.h"
#include "Rocket.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Rocket::Rocket()
{
	m_Spark.SetRocket(this);
	m_Smoke.SetRocket(this);
	m_Explosion.SetRocket(this);
}

Rocket::~Rocket()
{

}

void Rocket::Update(float m_fTimePassed)
{
	if (m_bExploded)
	{
		m_Explosion.Update(m_fTimePassed);
		if (m_Explosion.m_nDead == MAX_EXPLOSIONS)
		{
			GetThisApp()->m_aRockets.RemoveItem(this);
			return;
		}
	}
	if (m_fStartDelay > 0.0f)
	{
		m_fStartDelay -= m_fTimePassed;
		return;
	}
	if (!m_bSetOff)
	{
		lighterPos -= 180.0f*m_fTimePassed;
		if (lighterPos <= 0.0f)
		{
			m_fSmokeDelay = 0.3f;
			m_bSetOff = true;
			m_Spark.InitAll();
			m_Smoke.InitAll();
		}
	} else
	{
		if (m_fShootDelay > 0.0f)
		{
			m_fShootDelay -= m_fTimePassed;
			if (m_fShootDelay <= 0.0f)
			{
				m_fShootDelay = 0.0f;
				GetThisApp()->wavLaunch.PlayFile();
			}
		} else
		{
			z -= m_fShootSpeed*m_fTimePassed*(float)cos(DTOR(angle));
			y += m_fShootSpeed*m_fTimePassed*(float)sin(DTOR(angle));
			if (angle == blowAngle && m_fShootSpeed != 0.0f)
			{
				m_fShootSpeed = 0.0f;
				m_bExploded = true;
				m_Explosion.InitAll();

				GetThisApp()->m_fFlashAlpha = 0.4f;
	
				GetThisApp()->AddRocket();

				GetThisApp()->wavExplode.PlayFile();
				return;
			}

			angle += m_fTimePassed*10.0f;
			if (angle > blowAngle)
				angle = blowAngle;
		}

		m_Spark.Update(m_fTimePassed);
		if (m_fSmokeDelay < 0.0f)
			m_Smoke.Update(m_fTimePassed);
		else
			m_fSmokeDelay -= m_fTimePassed;
	}
}

void Rocket::RenderRocket()
{
	if (m_bExploded)
		return;

	glPushMatrix();
	glTranslatef(x, y, z);
	glColor3f(0.7f, 0.0f, 0.0f);
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glRotatef(180.0f+angle, 1.0f, 0.0f, 0.0f);
	glScalef(0.5f, 0.5f, 0.5f);
	GetThisApp()->m_pCracker->Display();
	glPopMatrix();
}

void Rocket::Render(float vx, float vy, float vz)
{
	glPushMatrix();


	if (!m_bSetOff)
	{
		// draw the lighter
		float lighterx, lightery;
		lighterx = (lighterPos/180.0f)*0.5f;
		lightery = 0.2f*(float)sin(DTOR(lighterPos));
		glDisable(GL_LIGHTING);
		glBindTexture(GL_TEXTURE_2D, GetThisApp()->textureParticle);

		glTranslatef(x, y, z);

		float string_cnt = 0.0f;
		float stringx, stringy;
		glDepthMask(FALSE);
		glBegin(GL_LINE_STRIP);
			glColor3f(0.2f, 0.2f, 0.2f);
			for (string_cnt = 0; string_cnt <= lighterPos; string_cnt += 1.0f)
			{
				stringx = (string_cnt/180.0f)*0.5f;
				stringy = 0.2f*(float)sin(DTOR(string_cnt));
				glVertex3f(stringx, stringy, 0.0f);
			}
		glEnd();
		glDepthMask(TRUE);

		glEnable(GL_BLEND);
		glEnable(GL_TEXTURE_2D);

		glColor3f(1.0f, 0.0f, 0.0f);
		DrawParticleLuce(lighterx, lightery, 0.0f, 0.02f, x+vx, y+vy, z+vz);
		
		glEnable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_BLEND);
	} else
	{
		if (!m_bExploded)
		{
			// draw all other particles here
			m_Spark.Render(vx, vy, vz);
			if (m_fSmokeDelay < 0.0f)
				m_Smoke.Render(vx, vy, vz);
		} else
		{
			m_Explosion.Render(vx,vy,vz);
		}
	}

	glPopMatrix();
}

