// SparkSystem.cpp: implementation of the SparkSystem class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CNY.h"
#include "Rocket.h"
#include "SparkSystem.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SparkSystem::SparkSystem()
{
}

SparkSystem::~SparkSystem()
{

}


void SparkSystem::InitAll()
{
	int cnt;
	nextblue = 0.0f;
	bluedir = 1.0f;
	for (cnt = 0; cnt < MAX_SPARKS; cnt++)
	{
		InitElement(cnt);
	}
}

inline void SparkSystem::InitElement(int index)
{
	sp[index].x = m_pRocket->x;
	sp[index].y = m_pRocket->y;
	sp[index].z = m_pRocket->z;
	sp[index].alpha = (((float)(rand()%1000)/1000.0f)*0.4f)+0.6f;

	sp[index].ydiff = -(((float)(rand()%1000)/1000.0f)*0.3f)-0.15f;
	sp[index].xdiff = (((float)(rand()%1000)/1000.0f)*0.3f)-0.15f;
	sp[index].zdiff = (((float)(rand()%1000)/1000.0f)*0.3f)+0.7f;
	sp[index].green = 0.3f;
	sp[index].blue = nextblue;
}

void SparkSystem::Render(float vx, float vy, float vz)
{
	int cnt;
	glDisable(GL_LIGHTING);
	glBindTexture(GL_TEXTURE_2D, GetThisApp()->textureParticle);
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_CULL_FACE);
	glDepthMask(FALSE);
	for (cnt = 0; cnt < MAX_SPARKS; cnt++)
	{
		glColor4f(1.0f, sp[cnt].green, sp[cnt].blue, sp[cnt].alpha);
		DrawParticleLuce(sp[cnt].x, sp[cnt].y, sp[cnt].z, 0.02f, vx, vy, vz);
	}
	glDepthMask(TRUE);
	glEnable(GL_LIGHTING);
	glEnable(GL_CULL_FACE);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

void SparkSystem::Update(float fTimePassed)
{
	int cnt;
	nextblue += bluedir*fTimePassed;
	if (nextblue >= 1.0f)
	{
		bluedir = -1.0f;
		nextblue = 1.0f;
	} else if (nextblue <= 0.0f)
	{
		bluedir = 1.0f;
		nextblue = 0.0f;
	}
	for (cnt = 0; cnt < MAX_SPARKS; cnt++)
	{
		sp[cnt].alpha -= fTimePassed*2.0f;
		if (sp[cnt].alpha <= 0.0f)
		{
			InitElement(cnt);
		} else
		{
			sp[cnt].green += 2.0f*fTimePassed;
			if (sp[cnt].green >= 1.0f)
				sp[cnt].green = 1.0f;
			sp[cnt].z += 3.0f*fTimePassed*sp[cnt].zdiff;
			sp[cnt].y += 3.0f*fTimePassed*sp[cnt].ydiff;
			sp[cnt].x += 3.0f*fTimePassed*sp[cnt].xdiff;
		}
	}
}

