// TextureLoader.cpp: implementation of the TextureLoader class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TextureLoader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TextureLoader::TextureLoader()
{

}

TextureLoader::~TextureLoader()
{

}


// tga loading from nehe
GLuint TextureLoader::LoadTGA(Pointer<SBFile> r_pFile, const String& r_sFileName)
{
	if (r_pFile == NULL)
		r_pFile = new DiskFile;

	GLubyte TGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0};
	GLubyte TGAcompare[12];
	GLubyte header[6];
	GLuint bytesPerPixel;
	GLuint imageSize;
	GLuint temp;
	GLuint type = GL_RGBA;

	if (!r_pFile->OpenFile(r_sFileName, FILE_BINARY_R))
	{
		GFC::GetDebugger()->ErrorOut("Unable to open file: " + r_sFileName + "\n");
		return 0;
	}

	if (r_pFile->ReadData(TGAcompare, 1, sizeof(TGAcompare)) != sizeof(TGAcompare))
	{
		GFC::GetDebugger()->ErrorOut("Unable to read TGA header: " + r_sFileName + "\n");
		return 0;
	}

	if (memcmp(TGAcompare, TGAheader, sizeof(TGAcompare)) != 0)
	{
		GFC::GetDebugger()->ErrorOut("Invalid file type: " + r_sFileName + "\n");
		return 0;
	}

	if (r_pFile->ReadData(header, 1, sizeof(header)) != sizeof(header))
	{
		GFC::GetDebugger()->ErrorOut("Unable to read image header: " + r_sFileName + "\n");
		return 0;
	}

	GLuint width, height, bpp;

	width = (header[1] * 256) + header[0];
	height = (header[3] * 256) + header[2];
	bpp = header[4];

	if (width < 0 || height < 0 || (bpp != 24 && bpp != 32))
		return 0;

	if (bpp == 24)
		type = GL_RGB;

	bytesPerPixel = bpp/8;
	imageSize = width * height * bytesPerPixel;

	GLubyte* imageData = (GLubyte*)malloc(imageSize);
	if (imageData == NULL)
		return 0;

	if (r_pFile->ReadData(imageData, 1, imageSize) != imageSize)
	{
		free(imageData);
		return 0;
	}

	r_pFile->CloseFile();

	// swap r and b values
	GLuint i;
	for (i = 0; i < int(imageSize); i += bytesPerPixel)
	{
		temp = imageData[i];
		imageData[i] = imageData[i+2];
		imageData[i+2] = temp;
	}

	GLuint texID;
	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);

	gluBuild2DMipmaps(GL_TEXTURE_2D, bytesPerPixel, width, height, type, GL_UNSIGNED_BYTE, imageData);

	free(imageData);

	return texID;
}


