/***************************************
* File    : DiskFile.cpp
* Author  : Shezan Baig 
* Purpose : Implementation of the DiskFile class
***************************************/

#include "stdafx.h"
#include "../Interfaces/SBFile.h"
#include "DiskFile.h"

IMPLEMENT_RTTI(DiskFile)

// default constructor
DiskFile::DiskFile()
{
	m_pFile = NULL;
}

// destructor
DiskFile::~DiskFile()
{
	CloseFile();
}


/////////////////////////////////////////////////////////////////////////////
// Methods inherited from SBFile interface
/////////////////////////////////////////////////////////////////////////////

// opens a file, returns true if successful
// parameters are same as fopen
bool DiskFile::OpenFile(String r_sFileName, String r_sMode)
{
	m_pFile = fopen(r_sFileName, r_sMode);
	return (m_pFile != NULL);
}

// closes the file, returns true if successful
bool DiskFile::CloseFile()
{
	if (m_pFile != NULL)
	{
		if (fclose(m_pFile) == 0)
		{
			m_pFile = NULL;
			return true;
		} else
			return false;
	} else
		return false;
}

// returns true if the file is opened
bool DiskFile::IsFileOpened()
{
	return (m_pFile != NULL);
}


// reads data from the file
// parameters similar to fread
size_t DiskFile::ReadData(void* buffer, size_t size, size_t count)
{
	return fread(buffer, size, count, m_pFile);
}

// writes data to the file
// parameters similar to fwrite
size_t DiskFile::WriteData(const void* buffer, size_t size, size_t count)
{
	return fwrite(buffer, size, count, m_pFile);
}

// returns the size of the file
// (file must be opened first)
size_t DiskFile::GetFileSize()
{
	// get current pos
	int pos = ftell(m_pFile);

	// go to the end of the file
	fseek(m_pFile, 0L, SEEK_END);

	// get the size
	size_t size = ftell(m_pFile);

	// move back to the old position
	fseek(m_pFile, pos-size, SEEK_CUR);

	// return the size
	return size;
}

// returns true if file pointer is at the end of the file
bool DiskFile::EndOfFile()
{
	return (feof(m_pFile) != 0);
}

// moves to a position in the file
// parameters similar to fseek()
bool DiskFile::Seek(long offset, int origin)
{
	if (fseek(m_pFile, offset, origin) == 0)
		return true;
	else
		return false;
}

// returns the position of the file pointer
// similar to ftell()
long DiskFile::GetPosition()
{
	return ftell(m_pFile);
}

