/***************************************
* File    : DiskFile.h
* Author  : Shezan Baig 
* Purpose : declares the DiskFile class
***************************************/

#ifndef _DISKFILE_H_INCLUDED
#define _DISKFILE_H_INCLUDED


/**********************************************
* Class       : DiskFile
* Author      : Shezan Baig 
* Purpose     : Represents a file on the disk
**********************************************/
class GFCFILES_API DiskFile : public virtual BaseObject,
								public SBFile
{
	DECLARE_RTTI(DiskFile)

	BEGIN_INTERFACE_LIST(DiskFile)
		INTERFACE(SBFile)
		INTERFACE(DiskFile)
	END_INTERFACE_LIST
	
public:
	// default constructor
	DiskFile();
	// destructor
	virtual ~DiskFile();

// Methods inherited from SBFile interface
	// opens a file, returns true if successful
	// parameters are same as fopen
	virtual bool OpenFile(String r_sFileName, String r_sMode);
	// closes the file, returns true if successful
	virtual bool CloseFile();
	// returns true if the file is opened
	virtual bool IsFileOpened();

	// reads data from the file
	// parameters similar to fread
	virtual size_t ReadData(void* buffer, size_t size, size_t count);
	// writes data to the file
	// parameters similar to fwrite
	virtual size_t WriteData(const void* buffer, size_t size, size_t count);

	// returns the size of the file
	// (file must be opened first)
	virtual size_t GetFileSize();
	// returns true if file pointer is at the end of the file
	virtual bool EndOfFile();

	// moves to a position in the file
	// parameters similar to fseek()
	virtual bool Seek(long offset, int origin);
	// returns the position of the file pointer
	// similar to ftell()
	virtual long GetPosition();

private:
	FILE* m_pFile;
};

typedef Pointer<DiskFile>	DiskFilePtr;

#endif // #ifndef _DISKFILE_H_INCLUDED
