/***************************************
* File    : GFCFiles.cpp
* Author  : Shezan Baig 
* Purpose : Defines the entry point for the GFCFiles DLL
***************************************/

#include "stdafx.h"

#ifdef WIN32
/**********************************************
* Function        : DllMain
* Author          : Shezan Baig 
* Purpose         : entry point for DLL
* Return Value    : standard Win32 DLL signature (refer to MSDN)
* Parameters      : standard Win32 DLL signature
**********************************************/
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

#endif // WIN32

#include "../Interfaces/SBFile.h"

/**********************************************
* Function        : WriteString
* Author          : Shezan Baig 
* Purpose         : writes a string into a file using a standardised format
* Return Value    : none
* Parameters      : the file, and the string
**********************************************/
GFCFILES_API void WriteString(Pointer<SBFile> r_pFile, const String& r_str)
{
	// write length of string
	short len = r_str.Length();
	r_pFile->WriteData(&len, sizeof(len), 1);
	if (len > 0)
		// write the string buffer
		r_pFile->WriteData((const char*)r_str, len, 1);
}

/**********************************************
* Function        : ReadString
* Author          : Shezan Baig 
* Purpose         : reads a string from a file using the standardised format
* Return Value    : none
* Parameters      : the file, and the location to store the string
**********************************************/
GFCFILES_API void ReadString(Pointer<SBFile> r_pFile, String& r_str)
{
	// read length of string
	short len;
	r_pFile->ReadData(&len, sizeof(len), 1);
	char* s = new char[len+1];
	if (len > 0)
	{
		// read the string buffer
		r_pFile->ReadData(s, len, 1);
	}
	s[len] = '\0';
	r_str = s;
	delete[] s;
}

