/***************************************
* File    : ResourceFile.cpp
* Author  : Shezan Baig 
* Purpose : implementation of the ResourceFile class.
***************************************/


#include "stdafx.h"
#include "../Interfaces/SBFile.h"
#include "ResourceFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


IMPLEMENT_RTTI(ResourceFile)

ResourceFile::ResourceFile()
{
	m_pData = NULL;
}

ResourceFile::~ResourceFile()
{
	CloseFile();
}


// methods from SBFile interface

// opens a file, returns true if successful
// parameters are same as fopen
bool ResourceFile::OpenFile(String r_sFileName, String r_sMode)
{
	CloseFile();

	// find the '\n' that separates resource name from resource type
	char s[MAX_PATH];
	strcpy(s, r_sFileName);
	char* ps = s;
	while (*ps != '\0')
	{
		if (*ps == '\n')
		{
			*ps = '\0';
			ps++;
			break;
		} else
			ps++;
	}

	if ((m_hResource = FindResource(m_hModule, s, ps)) != NULL)
	{
		if ((m_hGlobal = LoadResource(m_hModule, m_hResource)) != NULL)
		{
			m_nSize = SizeofResource(m_hModule, m_hResource);
			m_nFilePos = 0;
			m_pData = LockResource(m_hGlobal);

			if (m_pData == NULL)
				FreeResource(m_hGlobal);

			return (m_pData != NULL);
		} else
			return false;
	} else
		return false;
}

// closes the file, returns true if successful
bool ResourceFile::CloseFile()
{
	if (IsFileOpened())
	{
		UnlockResource(m_hGlobal);
		FreeResource(m_hGlobal);
		m_pData = NULL;

		return true;
	} else
		return false;
}

// returns true if the file is opened
bool ResourceFile::IsFileOpened()
{
	return (m_pData != NULL);
}


// reads data from the file
// parameters similar to fread
size_t ResourceFile::ReadData(void* buffer, size_t size, size_t count)
{
	if (!IsFileOpened())
		return 0;

	unsigned int cnt = 0;
	char* pBuf;
	char* pDest = (char*)buffer;
	while (cnt < count)
	{
		if ((m_nFilePos + (int)size) <= m_nSize)
		{
			pBuf = ((char*)m_pData) + m_nFilePos;
			memcpy(pDest, pBuf, size);
			pDest += (int)size;
			m_nFilePos += (int)size;
			cnt++;
		} else
			break;
	}

	return cnt;
}

// writes data to the file
// parameters similar to fwrite
size_t ResourceFile::WriteData(const void* buffer, size_t size, size_t count)
{
	// NOT SUPPORTED
	return 0;
}


// returns the size of the file
// (file must be opened first)
size_t ResourceFile::GetFileSize()
{
	return m_nSize;
}

// returns true if file pointer is at the end of the file
bool ResourceFile::EndOfFile()
{
	return (IsFileOpened() && m_nFilePos == m_nSize);
}


// moves to a position in the file
// parameters similar to fseek()
bool ResourceFile::Seek(long offset, int origin)
{
	if (!IsFileOpened())
		return false;

	if (origin == SEEK_SET)
	{
		if (offset < 0)
			m_nFilePos = 0;
		else if (offset > m_nSize)
			m_nFilePos = m_nSize;
		else
			m_nFilePos = offset;
	} else if (origin == SEEK_END)
	{
		if (offset > 0)
			m_nFilePos = m_nSize;
		else if (offset < -m_nSize)
			m_nFilePos = 0;
		else
			m_nFilePos = m_nSize + offset;
	} else if (origin == SEEK_CUR)
	{
		if (m_nFilePos+offset < 0)
			m_nFilePos = 0;
		else if (m_nFilePos+offset > m_nSize)
			m_nFilePos = m_nSize;
		else
			m_nFilePos += offset;
	}
	return true;
}

// returns the position of the file pointer
// similar to ftell()
long ResourceFile::GetPosition()
{
	return m_nFilePos;
}


