/***************************************
* File    : ResourceFile.h
* Author  : Shezan Baig 
* Purpose : interface for the ResourceFile class.
***************************************/

#if !defined(AFX_RESOURCEFILE_H__A0E46242_F966_11D5_90AA_C06B6BC10000__INCLUDED_)
#define AFX_RESOURCEFILE_H__A0E46242_F966_11D5_90AA_C06B6BC10000__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/**********************************************
* Class       : ResourceFile 
* Author      : Shezan Baig 
* Purpose     : This class allows users to read resources
*				in executable files as if the resources
*				were normal files.
**********************************************/
class GFCFILES_API ResourceFile : public virtual BaseObject,
						public SBFile
{
	DECLARE_RTTI(ResourceFile)

	BEGIN_INTERFACE_LIST(ResourceFile)
		INTERFACE(SBFile)
		INTERFACE(ResourceFile)
	END_INTERFACE_LIST
public:
	ResourceFile();
	virtual ~ResourceFile();

// methods from SBFile interface
	// opens a file, returns true if successful
	// parameters are same as fopen
	virtual bool OpenFile(String r_sFileName, String r_sMode);
	// closes the file, returns true if successful
	virtual bool CloseFile();
	// returns true if the file is opened
	virtual bool IsFileOpened();

	// reads data from the file
	// parameters similar to fread
	virtual size_t ReadData(void* buffer, size_t size, size_t count);
	// writes data to the file
	// parameters similar to fwrite
	virtual size_t WriteData(const void* buffer, size_t size, size_t count);

	// returns the size of the file
	// (file must be opened first)
	virtual size_t GetFileSize();
	// returns true if file pointer is at the end of the file
	virtual bool EndOfFile();

	// moves to a position in the file
	// parameters similar to fseek()
	virtual bool Seek(long offset, int origin);
	// returns the position of the file pointer
	// similar to ftell()
	virtual long GetPosition();

// Own methods
	void SetModule(HMODULE r_hModule)
	{
		m_hModule = r_hModule;
	}

private:
	HMODULE m_hModule;

	HGLOBAL m_hGlobal;
	HRSRC m_hResource;
	LPVOID m_pData;
	int m_nFilePos;
	int m_nSize;
};

#endif // !defined(AFX_RESOURCEFILE_H__A0E46242_F966_11D5_90AA_C06B6BC10000__INCLUDED_)
