/***************************************
* File    : ConfigReader.h
* Author  : Shezan Baig 
* Purpose : Declares the ConfigReader class
***************************************/

#ifndef _CONFIGREADER_H_INCLUDED
#define _CONFIGREADER_H_INCLUDED

/**********************************************
* Class       : ConfigReader
* Author      : Shezan Baig 
* Purpose     : Class which reads configuration files
*
* Note : Configuration files are ascii test files
*        which consist of lines of key and value pairs
*        in this format:
*                  key=value
**********************************************/
class GFCFRAMEWORK_API ConfigReader : public virtual BaseObject
{
	DECLARE_RTTI(ConfigReader)

	BEGIN_INTERFACE_LIST(ConfigReader)
	END_INTERFACE_LIST
public:
// constructors
	ConfigReader();
	ConfigReader(const String& r_sFileName);

// destructor
	virtual ~ConfigReader();

// methods
	inline void SetFileName(const String& r_sFileName);
	// returns true if the file is successfully opened
	inline bool OpenFile();
	// returns false if the file is already closed
	inline bool CloseFile();
	// returns true if the file is currently opened
	inline bool IsFileOpened() const;

	// returns true if the key is found
	bool ReadKey(const String& r_sKey, String& r_sValue);

private:
	FILE* m_pFile;
	String m_sFileName;
};
typedef Pointer<ConfigReader>		ConfigReaderPtr;

#endif // _CONFIGREADER_H_INCLUDED
