/***************************************
* File    : Exception.h
* Author  : Shezan Baig 
* Purpose : Declares the Exception class
***************************************/


#ifndef _EXCEPTION_H_INCLUDED
#define _EXCEPTION_H_INCLUDED

#include "RTTIClass.h"

/**********************************************
* Class       : Exception
* Author      : Shezan Baig 
* Purpose     : Holds exception information.
*				Specific exception classes can
*				be derived from here.
*
* Note : Exception classes do not use the garbage
*        collection mechanism defined in SBGC.h
*        Garbage collection for uncaught exceptions
*        are automatically be handled by GFC
*        internal library.
*        If you catch the exceptions in your program
*        you are responsible to delete the exception.
**********************************************/
class GFCFRAMEWORK_API Exception : public RTTI
{
	DECLARE_RTTI(Exception)

	BEGIN_INTERFACE_LIST(Exception)
	END_INTERFACE_LIST

public:
	Exception();
	Exception(const char* s, ...);
	virtual ~Exception();

	// returns a description of the exception
	String GetDescription() const;

protected:
	// sets the description of the exception
	void SetDescription(const String& s);

private:
	String m_sDesc;
};


#define EXCEPTION_LOCATION		(String)"\n\nFilename: " + __FILE__ + "\nLine: " + (String)__LINE__

#endif // #ifndef _EXCEPTION_H_INCLUDED
