/***************************************
* File    : GFCApplication.cpp
* Author  : Shezan Baig 
* Purpose : implementation of the GFCApplication class.
***************************************/

#include "StdAfx.h"
#include "Static_GFC.h"
#include "GFCApplication.h"

#include "FrameworkState.h"

// extern object declared in Static_GFC.cpp
extern FRAMEWORK_STATE	state;

GFCApplication::GFCApplication()
{
	// assign this application to the framework
	state.m_pApp = this;
}

GFCApplication::~GFCApplication()
{

}

bool GFCApplication::Init()
{
	return true;
}

bool GFCApplication::Run()
{
	while (GFC::GetWindow()->FlushMessages())
		PreIdle();

	return true;
}

void GFCApplication::OnExit()
{
}

void GFCApplication::OnIdle()
{
}

void GFCApplication::PreIdle()
{
	static unsigned long nPrevTime = GFC::GetTimer()->GetTickCount();
	static unsigned long nNowTime;

	nNowTime = GFC::GetTimer()->GetTickCount();

	m_fTimePassed = ((float)(nNowTime-nPrevTime))/1000.0f;

	nPrevTime = nNowTime;

	OnIdle();
	GFC::GetWindow()->InvalidateWindow();
}




/**********************************************
* Class       : DefaultDebugger 
* Author      : Shezan Baig 
* Purpose     : Provides a default debugger for applications that
*				do not use GFCMain
**********************************************/
class DefaultDebugger : public virtual BaseObject, public Debugger
{
	DECLARE_RTTI(DefaultDebugger)

	BEGIN_INTERFACE_LIST(DefaultDebugger)
		INTERFACE(Debugger)
		INTERFACE(DefaultDebugger)
	END_INTERFACE_LIST
public:
	DefaultDebugger()
	{
	}
	virtual ~DefaultDebugger()
	{
	}

// Implementation of Debugger
	virtual void DebugOut(const char* lpszString, ...) const
	{
		va_list vl;
		va_start(vl, lpszString);

		char s[1024];
		vsprintf(s, lpszString, vl);

		OutputDebugString(s);

		va_end(vl);
	}

	virtual void ErrorOut(const char* lpszString, ...) const
	{
		va_list vl;
		va_start(vl, lpszString);

		char s[1024];
		vsprintf(s, lpszString, vl);

		MessageBox(HWND_DESKTOP, s, "GFC Error Out", MB_ICONSTOP);

		va_end(vl);
	}
};


IMPLEMENT_RTTI(DefaultDebugger)


