/***************************************
* File    : GFCApplication.h
* Author  : Shezan Baig 
* Purpose : Defines the GFCApplication class
***************************************/

#ifndef _GFCAPPLICATION_H__INCLUDED
#define _GFCAPPLICATION_H__INCLUDED

#include "../Inline/String.inl"
#include "../Inline/SBArray.inl"

/**********************************************
* Class       : GFCApplication
* Author      : Shezan Baig 
* Purpose     : Base class which should be derived
*				in all applications
*
* Note : The object should be created on the stack,
*        therefore does not implement garbage collection.
*        At all times, there should be exactly *one*
*        instance of this class.
**********************************************/
class GFCFRAMEWORK_API GFCApplication
{
public:
	GFCApplication();
	virtual ~GFCApplication();

	virtual bool Init();
	virtual bool Run();
	virtual void OnIdle();
	virtual void OnExit();

// standard application information
// override these functions in your application
// to provide application specific information
	virtual String GetName() const
	{
		return "Unspecified";
	}
	virtual String GetCompany() const
	{
		return "Unspecified";
	}
	virtual String GetAuthors() const
	{
		return "\tUnspecified";
	}
	virtual String GetContactInfo() const
	{
		return "\tUnspecified";
	}

	// returns the amount of time passed (in seconds)
	// between each call to OnIdle()
	inline float GetTimePassed() const
	{
		return m_fTimePassed;
	}

	const String& GetCommandParams();

private:
	void PreIdle();
	float m_fTimePassed;
};

#endif // #ifndef _GFCAPPLICATION_H__INCLUDED

