/***************************************
* File    : LoadedLibrary.cpp
* Author  : Shezan Baig 
* Purpose : Implementation of LoadedLibrary class
***************************************/

#include "stdafx.h"
#include "LoadedLibrary.h"

#include "Static_GFC.h"

IMPLEMENT_RTTI(LoadedLibrary)

// Constructor
LoadedLibrary::LoadedLibrary()
{
	m_pHandle = NULL;
}

// Destructor
LoadedLibrary::~LoadedLibrary()
{
	if (m_pHandle != NULL)
	{
		// use OS specific functions to handle
		// library unloading
#ifdef WIN32
		FreeLibrary((HMODULE)m_pHandle);
#endif
		GFC::GetDebugger()->DebugOut("Unloaded library %s\n", (const char*)m_strFileName);

		m_pHandle = NULL;
	}
}


/**********************************************
* Function        : LoadedLibrary::Load
* Author          : Shezan Baig 
* Purpose         : Accepts a library filename and loads
*                   the library
* Return Value    : (bool) true if successful
* Parameters      : (const String& strFile) - File to load
**********************************************/
bool LoadedLibrary::Load(const String& strFile)
{
	m_strFileName = strFile;

	// use OS specific functions to handle
	// library unloading
#ifdef WIN32
	m_pHandle = (void*)LoadLibrary(strFile);
#endif

	if (m_pHandle != NULL)
		GFC::GetDebugger()->DebugOut("Loaded library %s\n", (const char*)m_strFileName);

	return (m_pHandle != NULL);
}

