/***************************************
* File    : Static_GFC.h
* Author  : Shezan Baig 
* Purpose : Defines the GFC class, which can be
*			used to obtain global information from
*			the framework
***************************************/

#ifndef _STATIC_GFC_H_INCLUDED
#define _STATIC_GFC_H_INCLUDED

/* Define the version for this build of GFC */
#define GFC_VERSION			1000


#include "GFCApplication.h"
#include "../Interfaces/Debugger.h"
#include "../Interfaces/PlatformInterfaces.h"

/**********************************************
* Class       : GFC
* Author      : Shezan Baig 
* Purpose     : Contains static functions for obtaining
*				information from the framework
**********************************************/
class GFCFRAMEWORK_API GFC  
{
public:
	static GFCApplication* GetApplication();

	static DebuggerCTPtr GetDebugger();
	static void SetDebugger(DebuggerPtr r_pDebugger);

	static const String& GetCommandLine();
	static void SetCommandLine(const String& r_sCommandLine);

	static bool LoadLibrary(const String& r_strFileName);
	static void UnloadLibrary(const String& r_strFileName);

	// makes sure cleanup order is correct
	static void CleanUp();

	/* platform interfaces */
	static bool LoadPlatform(const String& r_sConfigFile);
	static WindowPtr GetWindow();
	static InputPtr GetInput();
	static TimerPtr GetTimer();
};

#endif // #ifndef _STATIC_GFC_H_INCLUDED
