/***************************************
* File    : SBQueue.inl
* Author  : Shezan Baig 
* Purpose : Queue Module
*           Defines the Queue collection class
***************************************/

#ifndef _SBQUEUE_INL_INCLUDED
#define _SBQUEUE_INL_INCLUDED

#include "../Interfaces/Collection.h"

/**********************************************
* Class       : Queue<PtrType>
* Author      : Shezan Baig 
* Purpose     : Stores objects using a first in
*               first out method
**********************************************/
template <class PtrType>
class Queue : public virtual BaseObject,
				public Collection<PtrType>
{
private:
	// class queue_node represents a node in the queue
	class queue_node
	{
	public:
		queue_node()
		{
			next = NULL;
		}
		virtual ~queue_node()
		{
			if (next != NULL)
				delete next;
		}

		// stores a pointer to the object and a pointer
		// to the next queue node
		PtrType p;
		queue_node* next;
	};

public:
	Queue()
	{
		head = new queue_node;
		tail = head;
	}
	virtual ~Queue()
	{
		delete head;
	}

// Methods inherited from the Collection interface
	// inserts a pointer to the collection
	virtual void In(const PtrType& p)
	{
		queue_node* pnode = tail;
//		while (pnode->next != NULL)
//			pnode = pnode->next;

		pnode->next = new queue_node;
		pnode->next->p = p;
		tail = pnode->next;
	}

	// removes a pointer from the collection and returns it
	virtual PtrType Out()
	{
		queue_node* pnode = head->next;
		if (pnode == NULL)
			return NULL;

		if (tail == pnode)
			tail = head;

		PtrType p = pnode->p;
		head->next = pnode->next;
		// set the next pointer to NULL to prevent
		// it from being deleted in the destructor
		// of queue_node
		pnode->next = NULL;
		delete pnode;

		return p;
	}

	// returns a pointer to the next object to be removed by Out()
	virtual PtrType PeekOut() const
	{
		queue_node* pnode = head->next;
		if (pnode == NULL)
			return NULL;

		return pnode->p;
	}

	// returns the number of objects in the collection
	virtual int ItemCount() const
	{
		queue_node* pnode = head->next;
		int cnt = 0;
		while (pnode != NULL)
		{
			cnt++;
			pnode = pnode->next;
		}
		return cnt;
	}

private:
	queue_node* head;
	queue_node* tail;
};

#endif // _SBQUEUE_INL_INCLUDED
