/***************************************
* File    : SBStack.inl
* Author  : Shezan Baig 
* Purpose : Stack Module
*           Defines the Stack collection class
***************************************/

#ifndef _SBSTACK_INL_INCLUDED
#define _SBSTACK_INL_INCLUDED

#include "../Interfaces/Collection.h"

/**********************************************
* Class       : Stack<PtrType>
* Author      : Shezan Baig 
* Purpose     : Stores objects using a first
*               in last out method
**********************************************/
template <class PtrType>
class INLINE_API Stack : public virtual BaseObject,
							public Collection<PtrType>
{
private:
	// the stack_node class represents a node
	// in the stack
	class stack_node
	{
	public:
		stack_node(stack_node* pnext)
		{
			next = pnext;
		}
		virtual ~stack_node()
		{
			if (next != NULL)
				delete next;
		}

		// store a pointer to the object and to the next stack node
		PtrType p;
		stack_node* next;
	};

public:
	Stack()
	{
		top = NULL;
	}
	virtual ~Stack()
	{
		if (top != NULL)
			delete top;
	}

// Methods inherited from Collection interface
	// inserts a pointer to the collection
	virtual void In(const PtrType& p)
	{
		top = new stack_node(top);
		top->p = p;
	}

	// removes a pointer from the collection and returns it
	virtual PtrType Out()
	{
		if (top == NULL)
			return NULL;

		PtrType p = top->p;
		stack_node* pnode = top;
		top = pnode->next;
		// set the next node to NULL to avoid it from
		// being deleted in the destructor of stack_node
		pnode->next = NULL;
		delete pnode;

		return p;
	}

	// returns a pointer to the next object to be removed by Out()
	virtual PtrType PeekOut() const
	{
		if (top == NULL)
			return NULL;

		return top->p;
	}

	// returns the number of objects in the collection
	virtual int ItemCount() const
	{
		stack_node* pnode = top;
		int cnt = 0;
		while (pnode != NULL)
		{
			cnt++;
			pnode = pnode->next;
		}

		return cnt;
	}

private:
	stack_node* top;
};


#endif // #ifndef _SBSTACK_INL_INCLUDED
