/***************************************
* File    : String.inl
* Author  : Shezan Baig 
* Purpose : String module.
*			Defines string class which will be used
*			throughout the framework
***************************************/

#ifndef _STRING_INL_INCLUDED
#define _STRING_INL_INCLUDED

/**********************************************
* Class       : String
* Author      : Shezan Baig 
* Purpose     : Encapsulates functionality of a string
**********************************************/
class INLINE_API String
{
public:
// Constructor
	String()
	{
		m_psz = new char[1];
		m_psz[0] = '\0';
	}
	String(const String& r)
	{
		m_psz = new char[strlen(r.m_psz)+1];
		strcpy(m_psz, r.m_psz);
	}
	String(const char* r)
	{
		if (r == NULL)
		{
			m_psz = new char[1];
			m_psz[0] = '\0';
		} else
		{
			m_psz = new char[strlen(r)+1];
			strcpy(m_psz, r);
		}
	}
	String(char n)
	{
		m_psz = new char[2];
		m_psz[0] = n;
		m_psz[1] = '\0';
	}
	String(int n)
	{
		char t_psz[32];
		sprintf(t_psz, "%d", n);

		m_psz = new char[strlen(t_psz)+1];
		strcpy(m_psz, t_psz);
	}

// destructor
	virtual ~String()
	{
		delete[] m_psz;
	}

// friend operators
	friend String operator+(const String& l, const String& r);
	friend bool operator==(const String& l, const String& r);
	friend bool operator!=(const String& l, const String& r);
	friend bool operator<(const String& l, const String& r);
	friend bool operator<=(const String& l, const String& r);
	friend bool operator>(const String& l, const String& r);
	friend bool operator>=(const String& l, const String& r);

	
	const String operator=(const String& r)
	{
		if (&r != this && r.m_psz != this->m_psz)
		{
			// dont copy to itself
			delete[] m_psz;

			m_psz = new char[strlen(r.m_psz)+1];
			strcpy(m_psz, r.m_psz);
		}
		return *this;
	}

	operator const char*() const
	{
		return m_psz;
	}


	void TrimLeft()
	{
		char* pc = m_psz;
		while (*pc == ' ')
			pc++;
		*this = (String)pc;
	}

	void TrimRight()
	{
		char* pc = m_psz + strlen(m_psz) - 1;
		while (*pc == ' ')
			pc--;
		*(pc+1) = '\0';
		*this = (String)m_psz;
	}

	int Length() const
	{
		return strlen(m_psz);
	}

private:
	char* m_psz;
};

inline String operator+(const String& l, const String& r)
{
	char* t_psz = new char[strlen(l)+strlen(r)+1];
	strcpy(t_psz, l);
	strcat(t_psz, r);
	String s = t_psz;
	delete[] t_psz;
	return s;
}

inline bool operator==(const String& l, const String& r)
{
	return (strcmp(l.m_psz, r.m_psz) == 0);
}

inline bool operator!=(const String& l, const String& r)
{
	return (strcmp(l.m_psz, r.m_psz) != 0);
}

inline bool operator<(const String& l, const String& r)
{
	return (strcmp(l.m_psz, r.m_psz) < 0);
}

inline bool operator<=(const String& l, const String& r)
{
	return (strcmp(l.m_psz, r.m_psz) <= 0);
}

inline bool operator>(const String& l, const String& r)
{
	return (strcmp(l.m_psz, r.m_psz) > 0);
}

inline bool operator>=(const String& l, const String& r)
{
	return (strcmp(l.m_psz, r.m_psz) >= 0);
}


#endif // #ifndef _STRING_INL_INCLUDED
