/***************************************
* File    : SB3DObject.h
* Author  : Shezan Baig 
* Date    : 5/3/2001
* Purpose : Declares the SB3DObject interface
***************************************/

#ifndef _SB3DOBJECT_H_INCLUDED
#define _SB3DOBJECT_H_INCLUDED

/**********************************************
* Interface   : SB3DObject
* Author      : Shezan Baig 
* Date        : 5/3/2001
* Purpose     : interface to be used to manipulate 3D objects
**********************************************/
class SB3DObject : public virtual BaseObject
{
public:
	// load 3D object from a file, should return true if successful
	// the SBFilePtr interface can be supplied by the caller, or set
	// to NULL to use the default DiskFile
	virtual bool LoadFromFile(SBFilePtr r_pFile, const String& lpszFileName) = 0;
	// save 3D object to a file, should return true if successful
	// the SBFilePtr interface can be supplied by the caller, or set
	// to NULL to use the default DiskFile
	virtual bool SaveToFile(SBFilePtr r_pFile, const String& lpszFileName) const = 0;
	// call GL functions for drawing object
	virtual void Display() const = 0;
	// puts this 3D object in a display list
	// returns the name of the display list
	virtual unsigned int CreateDisplayList() = 0;

	// gets the sub-object with the specified name
	virtual Pointer<SB3DObject> GetObjectFromName(const String& lpszName) const = 0;
	// get the name of the this object
	virtual String GetName() const = 0;
	// sets the position of the object
	virtual void SetPosition(Float3CTPtr vPosition) = 0;
	// sets the direction to be pointed to
	virtual void SetDirection(Float3CTPtr vDirection) = 0;
	// gets the position of the object
	virtual Float3CTPtr GetPosition() const = 0;
	// gets the direction of the object
	virtual Float3CTPtr GetDirection() const = 0;

	// sets the rotation of the object
	virtual void SetRotation(Float3CTPtr afRotation) = 0;
	// gets the rotation of the object
	virtual Float3CTPtr GetRotation() const = 0;
	// sets the scale of the object
	virtual void SetScale(Float3CTPtr afScale) = 0;
	// gets the scale of the object
	virtual Float3CTPtr GetScale() const = 0;

	// adjusts vertex normals to smooth out edges with angles <=
	// the tolerance angle provided
	virtual void SmoothSurfaces(float fToleranceAngle) = 0;
};
typedef Pointer<SB3DObject>		SB3DObjectPtr;
typedef CTPointer<SB3DObject>	SB3DObjectCTPtr;


#endif // _SB3DOBJECT_H_INCLUDED
