/***************************************
* File    : SBFile.h
* Author  : Shezan Baig 
* Purpose : Declares the SBFile interface
***************************************/

#ifndef _SBFILE_H_INCLUDED
#define _SBFILE_H_INCLUDED

/**********************************************
* Interface   : SBFile
* Author      : Shezan Baig 
* Purpose     : Interface to be used for handling files
**********************************************/
class GFCFILES_API SBFile : public virtual BaseObject
{
public:
	// opens a file, returns true if successful
	// parameters are same as fopen
	virtual bool OpenFile(String r_sFileName, String r_sMode) = 0;
	// closes the file, returns true if successful
	virtual bool CloseFile() = 0;
	// returns true if the file is opened
	virtual bool IsFileOpened() = 0;

	// reads data from the file
	// parameters similar to fread
	virtual size_t ReadData(void* buffer, size_t size, size_t count) = 0;
	// writes data to the file
	// parameters similar to fwrite
	virtual size_t WriteData(const void* buffer, size_t size, size_t count) = 0;

	// returns the size of the file
	// (file must be opened first)
	virtual size_t GetFileSize() = 0;
	// returns true if file pointer is at the end of the file
	virtual bool EndOfFile() = 0;

	// moves to a position in the file
	// parameters similar to fseek()
	virtual bool Seek(long offset, int origin) = 0;
	// returns the position of the file pointer
	// similar to ftell()
	virtual long GetPosition() = 0;
};
typedef Pointer<SBFile>		SBFilePtr;

GFCFILES_API void WriteString(Pointer<SBFile> r_pFile, const String& r_str);
GFCFILES_API void ReadString(Pointer<SBFile> r_pFile, String& r_str);

#endif // _SBFILE_H_INCLUDED
