/***************************************
* File    : LoadCOB.h
* Author  : Shezan Baig 
* Date    : 5/3/2001
* Purpose : declarations for loading COB files
***************************************/

#ifndef _LOADCOB_H_INCLUDED
#define _LOADCOB_H_INCLUDED

/*
  Format for COB header

9 bytes	char	"Caligari "	Identifies Caligari file	
6 bytes	char	"V00.01"	Version number of file.  This document describes version 00.01 files	
1 byte	char	"A" for ASCII or "B" for binary	Specifies whether file is in ASCII format or binary format	
2 bytes	char	"LH" for little endian or "HL" for big endian	Intel based machines are little endian and currently all Caligari files are too.	
13 bytes	char	"             "	Leave as blank spaces	
1 byte	char	"\n"  Newline character	Newline	
*/



#define KNOWN_COB_MAGIC		"Caligari V00.01"

typedef struct
{
	char magic[15]; // should be "Caligari V00.01"
	char format; // ascii or binary
	char endian[2]; // always "LH"
	char whitespace[14]; // 13 spaces and a newline
} Tag_COBHeader;


/*
	Format for COB chunk header

Size	Type	Description	
4 bytes	char	Chunk type	
2 bytes	short	Major version	
2 bytes	short	Minor version	
4 bytes	long	Chunk ID	
4 bytes	long	Parent ID	
4 bytes	long	Number of bytes in chunk data	
*/

typedef struct
{
	char type[4];
	short vmajor;
	short vminor;
	int id;
	int parent;
	int size;
} Tag_COBChunkHeader;



class SBObject;

/*
	structure to map IDs to CSBObject object pointers
*/
typedef struct
{
	SBObject* m_pObject;
	int m_nID;
} Tag_ObjectIDMap;

/*
	structure to hold temporary information
	while loading COB file
*/
typedef struct
{
	std::vector < Tag_ObjectIDMap, std::allocator<Tag_ObjectIDMap> > m_aObjects;
} Tag_COBContext;



// return values for ProcessCOBChunk
#define		COBCHUNK_EOF			0	// end chunk
#define		COBCHUNK_UNKNOWN		1	// unknown chunk
#define		COBCHUNK_CONTINUE		2	// OK, continue to next chunk
#define		COBCHUNK_ERROR			3	// bad error, like eof reached before finished processing



#endif // #ifndef _LOADCOB_H_INCLUDED
