/***************************************
* File    : SBObjectClass.cpp
* Author  : Shezan Baig 
* Date    : 5/3/2001
* Purpose : implementation of a concrete class for SB3DObject interface
***************************************/

#include "stdafx.h"

#include "SBObject_Types.h"
#include "../Interfaces/SB3DObject.h"
#include "SBObjectClass.h"

SBOBJECT_API void Dummy() { }

IMPLEMENT_RTTI(SBObject)

// default constructor
SBObject::SBObject()
{
	m_aSubObjects.AddRef();

	m_nFaces = 0;
	m_nTextures = 0;
	m_nVertices = 0;
	m_nMaterials = 0;
	m_nNormals = 0;

	m_pFaces = NULL;
	m_pVertices = NULL;
	m_pMaterials = NULL;
	m_pNormals = NULL;
	m_pTextures = NULL;

	m_nDisplayList = 0;
	m_vPosition[0] = 0.0f;
	m_vPosition[1] = 0.0f;
	m_vPosition[2] = 0.0f;
	m_afRotation[0] = 0.0f;
	m_afRotation[1] = 0.0f;
	m_afRotation[2] = 0.0f;

	m_afScale[0] = 1.0f;
	m_afScale[1] = 1.0f;
	m_afScale[2] = 1.0f;

	m_avAxes[0][0] = 1.0f;
	m_avAxes[0][1] = 0.0f;
	m_avAxes[0][2] = 0.0f;
	m_avAxes[1][0] = 0.0f;
	m_avAxes[1][1] = 1.0f;
	m_avAxes[1][2] = 0.0f;
	m_avAxes[2][0] = 0.0f;
	m_avAxes[2][1] = 0.0f;
	m_avAxes[2][2] = 1.0f;

	m_sName = NULL;
	m_bCreatingDL = false;
}

// destructor
SBObject::~SBObject()
{
	// delete display list
	if (m_nDisplayList != 0)
	{
		glDeleteLists(m_nDisplayList, 1);
		m_nDisplayList = 0;
	}

	// delete everything
	SetFaceCount(0);
	SetMaterialCount(0);
	SetNormalCount(0);
	SetTextureCoordCount(0);
	SetVerticeCount(0);

	if (m_sName != NULL)
	{
		delete[] m_sName;
		m_sName = NULL;
	}
}



///////////////////////////////////////////////////////////////////////////////////////////////
// Methods inherited from SB3DObject interface
///////////////////////////////////////////////////////////////////////////////////////////////

// load 3D object from a file, should return true if successful
// the SBFilePtr interface can be supplied by the caller, or set
// to NULL to use the default DiskFile
bool SBObject::LoadFromFile(SBFilePtr r_pFile, const String& lpszFileName)
{
	if (r_pFile == NULL)
		r_pFile = new DiskFile;
	return LoadCOBFile(r_pFile, lpszFileName, GFC::GetDebugger());
}

// save 3D object to a file, should return true if successful
// the SBFilePtr interface can be supplied by the caller, or set
// to NULL to use the default DiskFile
bool SBObject::SaveToFile(SBFilePtr r_pFile, const String& lpszFileName) const
{
	return false;
}


// call GL functions for drawing object
void SBObject::Display() const
{
	glPushMatrix();


	if (!m_bCreatingDL)
	{
		// dont include transformations in the display list
		glScalef(m_afScale[SB_X], m_afScale[SB_Y], m_afScale[SB_Z]);

		glTranslatef(m_vPosition[SB_X], m_vPosition[SB_Y], m_vPosition[SB_Z]);

		// rotate object around axes
		glRotatef(m_afRotation[SB_Y], m_avAxes[SB_Y][SB_X], m_avAxes[SB_Y][SB_Y], m_avAxes[SB_Y][SB_Z]);
		glRotatef(m_afRotation[SB_Z], m_avAxes[SB_Z][SB_X], m_avAxes[SB_Z][SB_Y], m_avAxes[SB_Z][SB_Z]);
		glRotatef(m_afRotation[SB_X], m_avAxes[SB_X][SB_X], m_avAxes[SB_X][SB_Y], m_avAxes[SB_X][SB_Z]);
	}

	int nCounter;
	Pointer<SBObject> pObject;
	if (m_nDisplayList == 0)
	{
		int nLastTexture = -2;
		// disable texture if texture factory is null
		// if it is not null, GL_TEXTURE_2D will be enabled in the
		// LoadTexture function
//		if (pTextureFactory == NULL)
//			glDisable(GL_TEXTURE_2D);

		/*
		if (m_pParentObject == NULL)
		{
			glBegin(GL_LINES);
				glColor3f(1.0f, 0.0f, 0.0f);
				glVertex3f(0.0f, 0.0f, 0.0f);
				glVertex3f(5.5f*m_avAxes[SB_X][SB_X], 5.5f*m_avAxes[SB_X][SB_Y], 5.5f*m_avAxes[SB_X][SB_Z]);
				glColor3f(0.0f, 1.0f, 0.0f);
				glVertex3f(0.0f, 0.0f, 0.0f);
				glVertex3f(5.5f*m_avAxes[SB_Y][SB_X], 5.5f*m_avAxes[SB_Y][SB_Y], 5.5f*m_avAxes[SB_Y][SB_Z]);
				glColor3f(0.0f, 0.0f, 1.0f);
				glVertex3f(0.0f, 0.0f, 0.0f);
				glVertex3f(5.5f*m_avAxes[SB_Z][SB_X], 5.5f*m_avAxes[SB_Z][SB_Y], 5.5f*m_avAxes[SB_Z][SB_Z]);
			glEnd();

			glColor3f(1.0f, 1.0f, 1.0f);
		}
		//  */

//*
		bool blend_on = false;
		glDisable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

		GLfloat current_color[4];
		glGetFloatv(GL_CURRENT_COLOR, current_color);

		GLfloat emission[4];
		GLfloat specular[4];
		GLint shininess;
		Tag_Material material;

		glBegin(GL_TRIANGLES);
			for (nCounter = 0; nCounter < m_nFaces; nCounter++)
			{
				if (m_pMaterials != NULL)
				{
					SBrgba accum;

					material = m_pMaterials[m_pFaces[nCounter].m_nMaterial];
					accum[0] = current_color[0]*material.m_rgbaColor[0];
					accum[1] = current_color[1]*material.m_rgbaColor[1];
					accum[2] = current_color[2]*material.m_rgbaColor[2];
					accum[3] = current_color[3]*material.m_rgbaColor[3];

//					if (pTextureFactory != NULL && nLastTexture != m_pMaterials[m_pFaces[nCounter].m_nMaterial].m_nTexture)
//						pTextureFactory->LoadTexture(nLastTexture = m_pMaterials[m_pFaces[nCounter].m_nMaterial].m_nTexture);

					if (accum[3] < 1.0f)
					{
						if (!blend_on)
						{
							glEnd();
							glEnable(GL_BLEND);
							blend_on = true;
							glBegin(GL_TRIANGLES);
						}
					} else
					{
						if (blend_on)
						{
							glEnd();
							glDisable(GL_BLEND);
							blend_on = false;
							glBegin(GL_TRIANGLES);
						}
					}


					emission[0] = accum[0]*material.m_fAmbience;
					emission[1] = accum[1]*material.m_fAmbience;
					emission[2] = accum[2]*material.m_fAmbience;
					emission[3] = 1.0f;

					specular[0] = material.m_fSpecular;
					specular[1] = material.m_fSpecular;
					specular[2] = material.m_fSpecular;
					specular[3] = 1.0f;

					shininess = (GLint)(GLfloat)(material.m_fSpecular*128.0f);


					glColor4fv(accum);

					glMaterialfv(GL_FRONT, GL_EMISSION, emission);
					glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
					glMateriali(GL_FRONT, GL_SHININESS, shininess);
				}

				glNormal3fv(m_pNormals[m_pFaces[nCounter].m_nNormals[0]]);
				glTexCoord2fv(m_pTextures[m_pFaces[nCounter].m_nTextureCoords[0]]);
				glVertex3fv(m_pVertices[m_pFaces[nCounter].m_nVerts[0]]);

				glNormal3fv(m_pNormals[m_pFaces[nCounter].m_nNormals[1]]);
				glTexCoord2fv(m_pTextures[m_pFaces[nCounter].m_nTextureCoords[1]]);
				glVertex3fv(m_pVertices[m_pFaces[nCounter].m_nVerts[1]]);

				glNormal3fv(m_pNormals[m_pFaces[nCounter].m_nNormals[2]]);
				glTexCoord2fv(m_pTextures[m_pFaces[nCounter].m_nTextureCoords[2]]);
				glVertex3fv(m_pVertices[m_pFaces[nCounter].m_nVerts[2]]);

				if (m_pMaterials != NULL)
				{
					glColor4fv(current_color);
				}
			}
		glEnd();
		glDisable(GL_BLEND);
		blend_on = false;
// */

		if (!m_bCreatingDL)
		{
			// dont include subobjects in display list
			for (nCounter = 0; nCounter < m_aSubObjects.GetSize(); nCounter++)
			{
				pObject = m_aSubObjects[nCounter];
				pObject->Display();
			}
		}
	} else // this means (m_nDisplayList != 0)
	{
		glCallList(m_nDisplayList);

		// now call sub objects
		for (nCounter = 0; nCounter < m_aSubObjects.GetSize(); nCounter++)
		{
			pObject = m_aSubObjects[nCounter];
			pObject->Display();
		}
	}

	glPopMatrix();
}


// puts this 3D object in a display list
// returns the name of the display list
unsigned int SBObject::CreateDisplayList()
{
	if (m_nDisplayList != 0)
		// display list is already created
		return m_nDisplayList;

	m_bCreatingDL = true;

	GLuint t_dl = glGenLists(1);
	if (t_dl != GL_INVALID_VALUE && t_dl != GL_INVALID_OPERATION)
	{
		glNewList(t_dl, GL_COMPILE);
		Display();
		glEndList();

		m_nDisplayList = t_dl;

		// create display lists for child objects
		int nCounter;
		Pointer<SBObject> pObject;
		for (nCounter = 0; nCounter < m_aSubObjects.GetSize(); nCounter++)
		{
			pObject = m_aSubObjects[nCounter];
			pObject->CreateDisplayList();
		}
	}

	m_bCreatingDL = false;
	return m_nDisplayList;
}


// gets the sub-object with the specified name
SB3DObjectPtr SBObject::GetObjectFromName(const String& lpszName) const
{
	if (m_sName != NULL)
	{
		if (stricmp(lpszName, m_sName) == 0)
			return (SB3DObject*)this;
	}
	int nCounter;
	Pointer<SBObject> pObj;
	SB3DObjectPtr pRet;
	for (nCounter = 0; nCounter < m_aSubObjects.GetSize(); nCounter++)
	{
		pObj = m_aSubObjects[nCounter];
		if (pObj != NULL)
		{
			pRet = pObj->GetObjectFromName(lpszName);
			if (pRet != NULL)
				return pRet;
		}
	}

	return NULL;
}


// get the name of the this object
String SBObject::GetName() const
{
	return m_sName;
}


// sets the position of the object
void SBObject::SetPosition(Float3CTPtr vPosition)
{
	SBvertex f;
	vPosition->CopyTo(f);
	CopyVertex(m_vPosition, f);
}


// sets the direction to be pointed to
void SBObject::SetDirection(Float3CTPtr vDirection)
{
	SBvector f;
	vDirection->CopyTo(f);
	CopyVector(m_avAxes[SB_Z], f);
}

// gets the position of the object
Float3CTPtr SBObject::GetPosition() const
{
	return new Float3(m_vPosition);
}

// gets the direction of the object
Float3CTPtr SBObject::GetDirection() const
{
	return new Float3(m_avAxes[SB_Z]);
}

// sets the rotation of the object
void SBObject::SetRotation(Float3CTPtr afRotation)
{
	m_afRotation[SB_X] = (*afRotation)[SB_X];
	m_afRotation[SB_Y] = (*afRotation)[SB_Y];
	m_afRotation[SB_Z] = (*afRotation)[SB_Z];
}

// gets the rotation of the object
Float3CTPtr SBObject::GetRotation() const
{
	return new Float3(m_afRotation[SB_X],
						m_afRotation[SB_Y],
						m_afRotation[SB_Z]);
}

// sets the scale of the object
void SBObject::SetScale(Float3CTPtr afScale)
{
	m_afScale[SB_X] = (*afScale)[SB_X];
	m_afScale[SB_Y] = (*afScale)[SB_Y];
	m_afScale[SB_Z] = (*afScale)[SB_Z];
}

// gets the scale of the object
Float3CTPtr SBObject::GetScale() const
{
	return new Float3(m_afScale[SB_X],
						m_afScale[SB_Y],
						m_afScale[SB_Z]);
}


typedef struct
{
	int m_nFaceIndex; // index in m_pFaces
	int m_nVertIndex; // index in m_pFaces[m_nFaceIndex].m_pVerts
	bool m_bAddedToSmooth; // set to true when it is determined to be smoothed
	int m_nSmoothNormalIndex;
} Tag_TmpFaceUsed;

// adjusts vertex normals to smooth out edges with angles <=
// the tolerance angle provided
void SBObject::SmoothSurfaces(float fToleranceAngle)
{
	// list of all faces using the current vertex being processed
	std::vector < Tag_TmpFaceUsed, std::allocator<Tag_TmpFaceUsed> > t_aFaces;
	// normals
	std::vector < Tag_TmpVector, std::allocator<Tag_TmpVector> > t_aFaceNormals;
	// normals after smoothing
	// this will contain the final normals which will be transferred to
	// m_pNormals at the end of the function
	std::vector < Tag_TmpVector, std::allocator<Tag_TmpVector> > t_aSmoothNormals;

	// this will keep track of which normals have been added to vAccumNormal
	// to ensure that duplicate normals are not added
	std::vector < Tag_TmpVector, std::allocator<Tag_TmpVector> > t_aNormalsAdded;

	// initialise all faces to be 90 degrees facing upwards
	Tag_TmpVector temp;
	int nCounterFace;
	int nCounter;
	for (nCounterFace = 0; nCounterFace < GetFaceCount(); nCounterFace++)
	{
		CalcNormal(m_pVertices[m_pFaces[nCounterFace].m_nVerts[0]],
					m_pVertices[m_pFaces[nCounterFace].m_nVerts[1]],
					m_pVertices[m_pFaces[nCounterFace].m_nVerts[2]],
					temp.v);
		// see if temp.v is inside t_aFaceNormals
		for (nCounter = 0; nCounter < t_aFaceNormals.size(); nCounter++)
		{
			if (VectorsEqual(temp.v, t_aFaceNormals[nCounter].v))
				break;
		}
		if (nCounter == t_aFaceNormals.size())
		{
			// normal was not found in existing list
			t_aFaceNormals.push_back(temp);
		}
		m_pFaces[nCounterFace].m_nNormals[0] = nCounter;
		m_pFaces[nCounterFace].m_nNormals[1] = nCounter;
		m_pFaces[nCounterFace].m_nNormals[2] = nCounter;
	}


	float fAngle;
	Tag_TmpFaceUsed temp_face;
	SBvector vAccumNormal;
	int nSmoothCount; // keeps track of how many times vAccumNormal has been incremented
						// to get average normal, always divide by nSmoothCount

	float fSmoothCountInverse;
	int nCounterSmoothNormal;
	int nCounterFace2;

	bool bNormalAdded;
	int nCounterNormalsAdded;

	// loop thru all the vertices
	for (nCounter = 0; nCounter < GetVerticeCount(); nCounter++)
	{
		// fill up t_aFaces
		for (nCounterFace = 0; nCounterFace < GetFaceCount(); nCounterFace++)
		{
			if (m_pFaces[nCounterFace].m_nVerts[0] == nCounter
				|| m_pFaces[nCounterFace].m_nVerts[1] == nCounter
				|| m_pFaces[nCounterFace].m_nVerts[2] == nCounter)
			{
				// its using this vertice, add it to the list
				temp_face.m_bAddedToSmooth = false;
				temp_face.m_nSmoothNormalIndex = -1;
				temp_face.m_nFaceIndex = nCounterFace;
				temp_face.m_nVertIndex = (m_pFaces[nCounterFace].m_nVerts[0] == nCounter) ? 0 :
										 (m_pFaces[nCounterFace].m_nVerts[1] == nCounter) ? 1 : 2;
				t_aFaces.push_back(temp_face);
			}
		}

		for (nCounterFace = 0; nCounterFace < t_aFaces.size(); nCounterFace++)
		{
			nSmoothCount = 0;
			vAccumNormal[0] = 0.0f;
			vAccumNormal[1] = 0.0f;
			vAccumNormal[2] = 0.0f;

			for (nCounterFace2 = 0; nCounterFace2 < t_aFaces.size(); nCounterFace2++)
			{
				if (nCounterFace != nCounterFace2)
				{
					// make sure the normals are not the same
					if (m_pFaces[t_aFaces[nCounterFace].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace].m_nVertIndex] !=
						m_pFaces[t_aFaces[nCounterFace2].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace2].m_nVertIndex])
					{
						// get the angle between them
						fAngle = AngleBetween(t_aFaceNormals[m_pFaces[t_aFaces[nCounterFace].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace].m_nVertIndex]].v,
												t_aFaceNormals[m_pFaces[t_aFaces[nCounterFace2].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace2].m_nVertIndex]].v);

						// check against tolerance angle
						if (fAngle < fToleranceAngle)
						{
							// edge cannot be tolerated, smooth it out

							CopyVector(temp.v, t_aFaceNormals[m_pFaces[t_aFaces[nCounterFace].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace].m_nVertIndex]].v);

							// check if normal for face 1 has been added
							bNormalAdded = false;
							for (nCounterNormalsAdded = 0; nCounterNormalsAdded < t_aNormalsAdded.size(); nCounterNormalsAdded++)
							{
								if (VectorsEqual(temp.v, t_aNormalsAdded[nCounterNormalsAdded].v))
								{
									bNormalAdded = true;
									break;
								}
							}

							// accumulate the normal vector if it hasnt been added
							if (bNormalAdded == false)
							{
								nSmoothCount++;
								vAccumNormal[0] += temp.v[0];
								vAccumNormal[1] += temp.v[1];
								vAccumNormal[2] += temp.v[2];

								// update t_aNormalsAdded
								t_aNormalsAdded.push_back(temp);
							}


							// do the same thing for face 2
							CopyVector(temp.v, t_aFaceNormals[m_pFaces[t_aFaces[nCounterFace2].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace2].m_nVertIndex]].v);

							// check if normal for face 2 has been added
							bNormalAdded = false;
							for (nCounterNormalsAdded = 0; nCounterNormalsAdded < t_aNormalsAdded.size(); nCounterNormalsAdded++)
							{
								if (VectorsEqual(temp.v, t_aNormalsAdded[nCounterNormalsAdded].v))
								{
									bNormalAdded = true;
									break;
								}
							}

							// accumulate the normal vector if it hasnt been added
							if (bNormalAdded == false)
							{
								nSmoothCount++;
								vAccumNormal[0] += temp.v[0];
								vAccumNormal[1] += temp.v[1];
								vAccumNormal[2] += temp.v[2];

								// update t_aNormalsAdded
								t_aNormalsAdded.push_back(temp);
							}
						}
					}
				}
			}

			if (nSmoothCount > 0)
			{
				fSmoothCountInverse = 1.0f / ((float)nSmoothCount);
				vAccumNormal[0] *= fSmoothCountInverse;
				vAccumNormal[1] *= fSmoothCountInverse;
				vAccumNormal[2] *= fSmoothCountInverse;
				ReduceToUnit(vAccumNormal);

				// add accumNormal to the smooth normal list if it is not
				// already there
				for (nCounterSmoothNormal = 0; nCounterSmoothNormal < t_aSmoothNormals.size(); nCounterSmoothNormal++)
				{
					if (VectorsEqual(vAccumNormal, t_aSmoothNormals[nCounterSmoothNormal].v))
						break;
				}
				if (nCounterSmoothNormal == t_aSmoothNormals.size())
				{
					// its not found, so add it
					CopyVector(temp.v, vAccumNormal);
					t_aSmoothNormals.push_back(temp);
				}

				t_aFaces[nCounterFace].m_nSmoothNormalIndex = nCounterSmoothNormal;
				t_aFaces[nCounterFace].m_bAddedToSmooth = true;
			}

			// remove everything from t_aNormalsAdded
			while (t_aNormalsAdded.size() > 0)
				t_aNormalsAdded.erase(t_aNormalsAdded.begin());
		}

		
		for (nCounterFace = 0; nCounterFace < t_aFaces.size(); nCounterFace++)
		{
			if (t_aFaces[nCounterFace].m_bAddedToSmooth)
			{
				m_pFaces[t_aFaces[nCounterFace].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace].m_nVertIndex] = t_aFaces[nCounterFace].m_nSmoothNormalIndex;
			}
		}

		// for the faces that were not included to be smoothed, add their
		// normals into t_aSmoothNormals anyway, which will then once and
		// for all be copied to m_pNormals
		for (nCounterFace = 0; nCounterFace < t_aFaces.size(); nCounterFace++)
		{
			if (!t_aFaces[nCounterFace].m_bAddedToSmooth)
			{
				temp = t_aFaceNormals[m_pFaces[t_aFaces[nCounterFace].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace].m_nVertIndex]];
				for (nCounterSmoothNormal = 0; nCounterSmoothNormal < t_aSmoothNormals.size(); nCounterSmoothNormal++)
				{
					if (VectorsEqual(temp.v, t_aSmoothNormals[nCounterSmoothNormal].v))
						break;
				}
				if (nCounterSmoothNormal == t_aSmoothNormals.size())
					// its not found, so add it
					t_aSmoothNormals.push_back(temp);
				m_pFaces[t_aFaces[nCounterFace].m_nFaceIndex].m_nNormals[t_aFaces[nCounterFace].m_nVertIndex] = nCounterSmoothNormal;
			}
		}

		// remove everything from t_aFaces
		while (t_aFaces.size() > 0)
			t_aFaces.erase(t_aFaces.begin());
	}


	// all edges have been smoothed, now transfer contents of t_aSmoothNormals
	// to m_pNormals
	SetNormalCount(t_aSmoothNormals.size());
	for (nCounter = 0; nCounter < GetNormalCount(); nCounter++)
	{
		CopyVector(m_pNormals[nCounter], t_aSmoothNormals[nCounter].v);
	}



	// remove everything from normal vectors
	while (t_aFaceNormals.size() > 0)
		t_aFaceNormals.erase(t_aFaceNormals.begin());
	while (t_aSmoothNormals.size() > 0)
		t_aSmoothNormals.erase(t_aSmoothNormals.begin());


	// call sub objects
	Pointer<SBObject> pObj;
	for (nCounter = 0; nCounter < m_aSubObjects.GetSize(); nCounter++)
	{
		pObj = m_aSubObjects[nCounter];
		if (pObj != NULL)
			pObj->SmoothSurfaces(fToleranceAngle);
	}
}



inline bool SBObject::ShareEdges(int nFace1, int nFace2)
{
	int vertcnt = 0;
	if (m_pFaces[nFace1].m_nVerts[0] == m_pFaces[nFace2].m_nVerts[0]
		|| m_pFaces[nFace1].m_nVerts[0] == m_pFaces[nFace2].m_nVerts[1]
		|| m_pFaces[nFace1].m_nVerts[0] == m_pFaces[nFace2].m_nVerts[2])
		vertcnt++;
	if (m_pFaces[nFace1].m_nVerts[1] == m_pFaces[nFace2].m_nVerts[0]
		|| m_pFaces[nFace1].m_nVerts[1] == m_pFaces[nFace2].m_nVerts[1]
		|| m_pFaces[nFace1].m_nVerts[1] == m_pFaces[nFace2].m_nVerts[2])
		vertcnt++;
	if (m_pFaces[nFace1].m_nVerts[2] == m_pFaces[nFace2].m_nVerts[0]
		|| m_pFaces[nFace1].m_nVerts[2] == m_pFaces[nFace2].m_nVerts[1]
		|| m_pFaces[nFace1].m_nVerts[2] == m_pFaces[nFace2].m_nVerts[2])
		vertcnt++;

	return (vertcnt > 1);
}

