/***************************************
* File    : Win32Impl.cpp
* Author  : Shezan Baig 
* Date    : 2/3/2001
* Purpose : Defines the entry point for the DLL application.
			Implementation of CWin32Impl class
***************************************/

#include "stdafx.h"
#include "Win32Impl.h"

#ifndef GET_X_LPARAM
#define GET_X_LPARAM(lp)   ((int)(short)LOWORD(lp))
#endif // GET_X_LPARAM

#ifndef GET_Y_LPARAM
#define GET_Y_LPARAM(lp)   ((int)(short)HIWORD(lp))
#endif // GET_Y_LPARAM

HANDLE g_hModule = NULL;

/**********************************************
* Function      : DllMain
* Author        : Shezan Baig 
* Date          : 2/3/2001
* Purpose       : entry point for the DLL application
* Return Value  : BOOL (always TRUE)
* Parameters    : None
**********************************************/
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	g_hModule = hModule;
    return TRUE;
}


IMPLEMENT_RTTI(CWin32Impl)

Pointer<CWin32Impl> CWin32Impl::OneAndOnlyWin32Impl = NULL;

#include <stdio.h>

LONG WINAPI WndProc (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static HBRUSH brBlack = NULL;
	static HFONT hNewFont = NULL;
	static HFONT hOldFont = NULL;

    switch (msg)
	{
    case WM_CREATE:
		CWin32Impl::OneAndOnlyWin32Impl->m_hDC = GetDC(hwnd);

		hNewFont = CreateFont(12, 0, 0, 0, 400, 0, 0, 0, DEFAULT_CHARSET,
			OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
			DEFAULT_PITCH, "Impact");
		hOldFont = (HFONT)SelectObject(CWin32Impl::OneAndOnlyWin32Impl->m_hDC, hNewFont);
		DeleteObject(hOldFont);


		CWin32Impl::OneAndOnlyWin32Impl->SetDCPixelFormat(CWin32Impl::OneAndOnlyWin32Impl->m_hDC);

		GFC::GetDebugger()->DebugOut("Creating OpenGL context...\n");
		CWin32Impl::OneAndOnlyWin32Impl->m_hGLRC = wglCreateContext (CWin32Impl::OneAndOnlyWin32Impl->m_hDC);

		CWin32Impl::OneAndOnlyWin32Impl->CaptureRC();

		if (wglUseFontOutlines(wglGetCurrentDC(), 0, 255, 1000, 0.0f, 0.25f,  
			        WGL_FONT_POLYGONS, NULL) == FALSE)
		{
			GFC::GetDebugger()->DebugOut("****** ERROR WITH wglUseFont\n");
		}

		if (CWin32Impl::OneAndOnlyWin32Impl->m_pOpenGL != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pOpenGL->OnInitialiseRC();

		CWin32Impl::OneAndOnlyWin32Impl->ReleaseRC();
		brBlack = ::CreateSolidBrush(RGB(0,0,0));

//		ShowCursor(FALSE);
		return 0;
	case WM_CLOSE:
		return 0;
	case WM_DESTROY:
		ShowCursor(TRUE);

		if (CWin32Impl::OneAndOnlyWin32Impl->m_bFullScreenMode)
		{
			ChangeDisplaySettings(NULL,0);					// If So Switch Back To The Desktop
			CWin32Impl::OneAndOnlyWin32Impl->m_bFullScreenMode = false;
		}


		GFC::GetDebugger()->DebugOut("Deleting OpenGL context...\n");

		CWin32Impl::OneAndOnlyWin32Impl->CaptureRC();

		if (CWin32Impl::OneAndOnlyWin32Impl->m_pOpenGL != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pOpenGL->OnDeleteRC();


		glDeleteLists(1000, 256);
		glFinish();
		CWin32Impl::OneAndOnlyWin32Impl->ReleaseRC();


		wglDeleteContext (CWin32Impl::OneAndOnlyWin32Impl->m_hGLRC);
		ReleaseDC (hwnd, CWin32Impl::OneAndOnlyWin32Impl->m_hDC);

		if (CWin32Impl::OneAndOnlyWin32Impl->m_hPalette != NULL)
			DeleteObject(CWin32Impl::OneAndOnlyWin32Impl->m_hPalette);

		if (brBlack != NULL)
			DeleteObject(brBlack);

		GFC::GetDebugger()->DebugOut("Posting quit message.\n");
		PostQuitMessage(0);
		return 0;
    case WM_SIZE:
		CWin32Impl::OneAndOnlyWin32Impl->CaptureRC();
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pOpenGL != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pOpenGL->OnWindowSizeChange((int)LOWORD(lParam), (int)HIWORD(lParam));
		CWin32Impl::OneAndOnlyWin32Impl->ReleaseRC();
		return 0;

	case WM_KEYDOWN:
		//*
		if ((int)wParam == VK_ESCAPE)
		{
			ShowWindow(hwnd, SW_HIDE);
			UpdateWindow(hwnd);
			DestroyWindow(hwnd);
		}
//		*/
		if (!CWin32Impl::OneAndOnlyWin32Impl->m_bKeyState[(int)wParam])
		{
			CWin32Impl::OneAndOnlyWin32Impl->m_bKeyState[(int)wParam] = true;
			if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
				CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->KeyboardFunction(KB_KEYDOWN, (int)wParam);
		}
		return 0;
	case WM_KEYUP:
		CWin32Impl::OneAndOnlyWin32Impl->m_bKeyState[(int)wParam] = false;
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->KeyboardFunction(KB_KEYUP, (int)wParam);
		return 0;

	case WM_LBUTTONDOWN:
		CWin32Impl::OneAndOnlyWin32Impl->m_bMouseState[LEFT_BUTTON] = true;
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_LBUTTONDOWN, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_RBUTTONDOWN:
		CWin32Impl::OneAndOnlyWin32Impl->m_bMouseState[RIGHT_BUTTON] = true;
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_RBUTTONDOWN, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_LBUTTONUP:
		CWin32Impl::OneAndOnlyWin32Impl->m_bMouseState[LEFT_BUTTON] = false;
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_LBUTTONUP, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_RBUTTONUP:
		CWin32Impl::OneAndOnlyWin32Impl->m_bMouseState[RIGHT_BUTTON] = false;
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_RBUTTONUP, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_LBUTTONDBLCLK:
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_LBUTTONDBLCLK, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_RBUTTONDBLCLK:
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_RBUTTONDBLCLK, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_MBUTTONDOWN:
		CWin32Impl::OneAndOnlyWin32Impl->m_bMouseState[MIDDLE_BUTTON] = true;
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_MBUTTONDOWN, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_MBUTTONUP:
		CWin32Impl::OneAndOnlyWin32Impl->m_bMouseState[MIDDLE_BUTTON] = false;
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_MBUTTONUP, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_MBUTTONDBLCLK:
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
			CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_MBUTTONDBLCLK, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	case WM_MOUSEMOVE:
		if (!CWin32Impl::OneAndOnlyWin32Impl->m_bSettingMousePos)
		{
			if (CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget != NULL)
				CWin32Impl::OneAndOnlyWin32Impl->m_pInputTarget->MouseFunction(MOUSE_MOVE, GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		} else // this means setting mouse position, dont fire event
		{
			CWin32Impl::OneAndOnlyWin32Impl->m_bSettingMousePos = false;
		}
		return 0;
	
	case WM_PAINT:
		PAINTSTRUCT ps;
		BeginPaint(hwnd, &ps);
		if (CWin32Impl::OneAndOnlyWin32Impl->m_pOpenGL != NULL)
		{
			CWin32Impl::OneAndOnlyWin32Impl->CaptureRC();
			CWin32Impl::OneAndOnlyWin32Impl->m_pOpenGL->OnDrawScene();
			SwapBuffers(CWin32Impl::OneAndOnlyWin32Impl->m_hDC);
			CWin32Impl::OneAndOnlyWin32Impl->ReleaseRC();
		} else
		{
			RECT rc;
			GetClientRect(hwnd, &rc);
			FillRect(CWin32Impl::OneAndOnlyWin32Impl->m_hDC, &rc, brBlack);
		}
		EndPaint(hwnd, &ps);
		return 0;
	}
	return DefWindowProc(hwnd, msg, wParam, lParam);
}


// functions from IWindow
bool CWin32Impl::CreateWindow(const String& lpszTitle)
{
    WNDCLASS wc;

	GFC::GetDebugger()->DebugOut("Initing WNDCLASS\n");

    wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS | CS_OWNDC;
    wc.lpfnWndProc = (WNDPROC) WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = (HINSTANCE)g_hModule;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "GameEng_Win32Impl";

	GFC::GetDebugger()->DebugOut("Registering WNDCLASS\n");

    RegisterClass (&wc);

	GFC::GetDebugger()->DebugOut("Initialising Input Data\n");

	InitialiseInputData();

	GFC::GetDebugger()->DebugOut("Calling CreateWindowEx...\n");

    m_hWnd = CreateWindowEx(/*WS_EX_TOPMOST*/0, wc.lpszClassName, lpszTitle,
        WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
        CW_USEDEFAULT, CW_USEDEFAULT, 200, 200,
        HWND_DESKTOP, NULL, (HINSTANCE)g_hModule, NULL);

	GFC::GetDebugger()->DebugOut((m_hWnd == NULL) ? "m_hWnd is NULL\n" : "m_hWnd not NULL\n");

	return (m_hWnd != NULL);
}

bool CWin32Impl::CloseWindow()
{
	if (m_hWnd != NULL)
		return (DestroyWindow(m_hWnd) == TRUE);

	return true;
}

bool CWin32Impl::ShowWindow()
{
	::ShowWindow(m_hWnd, SW_RESTORE);
	UpdateWindow(m_hWnd);
	SetFocus(m_hWnd);

	return true;
}

// fullscreen code taken from Gamedev.net
bool ChangeToFullScreen()
{
	DEVMODE dmSettings;									// Device Mode variable

	memset(&dmSettings,0,sizeof(dmSettings));			// Makes Sure Memory's Cleared

	// Get current settings -- This function fills our the settings
	// This makes sure NT and Win98 machines change correctly
	if(!EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&dmSettings))
	{
		GFC::GetDebugger()->ErrorOut("ERROR:Unable to enum display settings!\n");
		return false;
	}

	dmSettings.dmPelsWidth	= 800;				// Selected Screen Width
	dmSettings.dmPelsHeight	= 600;			// Selected Screen Height
	
	// This function actually changes the screen to full screen
	// CDS_FULLSCREEN Gets Rid Of Start Bar.
	// We always want to get a result from this function to check if we failed
	int result = ChangeDisplaySettings(&dmSettings,CDS_FULLSCREEN);	

	// Check if we didn't recieved a good return message From the function
	if(result != DISP_CHANGE_SUCCESSFUL)
	{
		GFC::GetDebugger()->ErrorOut("ERROR:Display settings not supported!\n");
		return false;
	}

	return true;
}


bool CWin32Impl::SetFullScreen()
{
	if (m_bFullScreenMode)
		return true;

	glFinish();

	DWORD dwCurrent;
	dwCurrent = GetWindowLong(GetHwnd(), GWL_STYLE);
	DWORD dwStyle = WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
	if (dwCurrent & WS_VISIBLE)
		dwStyle |= WS_VISIBLE;
	SetWindowLong(GetHwnd(), GWL_STYLE, dwStyle);
	SetWindowPos(m_hWnd, HWND_TOPMOST,
				0, 0,
				GetSystemMetrics(SM_CXSCREEN),
				GetSystemMetrics(SM_CYSCREEN),
				0);

	m_bFullScreenMode = ChangeToFullScreen();
	return m_bFullScreenMode;
}

bool CWin32Impl::SetWindowRect(int left, int top, int right, int bottom)
{
	glFinish();

	if (m_bFullScreenMode)
	{
		ChangeDisplaySettings(NULL,0);					// If So Switch Back To The Desktop
		m_bFullScreenMode = false;
	}

	DWORD dwCurrent;
	dwCurrent = GetWindowLong(GetHwnd(), GWL_STYLE);
	DWORD dwStyle = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
	if (dwCurrent & WS_VISIBLE)
		dwStyle |= WS_VISIBLE;
	SetWindowLong(GetHwnd(), GWL_STYLE, dwStyle);

	MoveWindow(m_hWnd, left, top,
				right-left, bottom-top,
				TRUE);
	return true;
}

bool CWin32Impl::FlushMessages()
{
	MSG msg;
	bool bContinue = true;

	while (bContinue && PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) != 0)
	{
	    if (GetMessage (&msg, NULL, 0, 0))
		{
			TranslateMessage (&msg);
		    DispatchMessage (&msg);
		} else
			bContinue = false;
	}

    return bContinue;
}

// invalidates the window and forces repaint
void CWin32Impl::InvalidateWindow()
{
	::InvalidateRect(m_hWnd, NULL, FALSE);
	::UpdateWindow(m_hWnd);
}

// sets the pointer to the IOpenGL interface for callback functions
void CWin32Impl::SetOpenGL(OGLPtr lpOpenGL)
{
	m_pOpenGL = lpOpenGL;
}



// functions from IInput
bool CWin32Impl::IsKeyDown(int nKeyCode) const
{
	return m_bKeyState[nKeyCode];
}

bool CWin32Impl::IsLButtonDown() const
{
	return m_bMouseState[LEFT_BUTTON];
}

bool CWin32Impl::IsRButtonDown() const
{
	return m_bMouseState[RIGHT_BUTTON];
}

void CWin32Impl::SetInputTarget(InputTargetPtr lpInputTarget)
{
	m_pInputTarget = lpInputTarget;
}


// sets the position of the mouse to the ones specified by nX and xY
// if nX and nY are -1, then the mouse is centered in the window, and
// nX and nY are updated accordingly
void CWin32Impl::SetMousePosition(int& nX, int& nY)
{
	if (nX == -1 && nY == -1)
	{
		RECT rc;
		::GetWindowRect(m_hWnd, &rc);
		nX = (rc.left + rc.right)/2;
		nY = (rc.top + rc.bottom)/2;
	}

	// set this flag to true, so that the mouse move event for the
	// application is not fired
	m_bSettingMousePos = true;

	// call win32 function to set the position
	SetCursorPos(nX, nY);
}

/**********************************************
* Function      : CWin32Impl::SetDCPixelFormat
* Author        : Shezan Baig 
* Date          : 2/3/2001
* Purpose       : prepare the device context with the correct
*				  pixel format for creating the rendering context
* Return Value  : None
* Parameters    : HDC hdc = Device context of window
**********************************************/
void CWin32Impl::SetDCPixelFormat(HDC hdc)
{
	HANDLE hHeap;
	int nColors, i;
	LPLOGPALETTE lpPalette;
	BYTE byRedMask, byGreenMask, byBlueMask;

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof (PIXELFORMATDESCRIPTOR),				// Size of this structure
		1,											// Version number
		PFD_DRAW_TO_WINDOW |						// Flags
		PFD_SUPPORT_OPENGL |
		PFD_GENERIC_ACCELERATED|
		PFD_SWAP_EXCHANGE|
		PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,								// RGBA pixel values
		32,											// 24-bit color
		0, 0, 0, 0, 0, 0,							// Don't care about these
		0, 0,										// No alpha buffer
		0, 0, 0, 0, 0,								// No accumulation buffer
		32,											// 32-bit depth buffer
		8,											// No stencil buffer
		0,											// No auxiliary buffers
		PFD_MAIN_PLANE,								// Layer type
		0,											// Reserved (must be 0)
		0, 0, 0										// No layer masks
	};

	int nPixelFormat;

	GFC::GetDebugger()->DebugOut("Choosing pixel format\n");

	nPixelFormat = ChoosePixelFormat (hdc, &pfd);

	GFC::GetDebugger()->DebugOut("Setting pixel format\n");

	SetPixelFormat (hdc, nPixelFormat, &pfd);

	if (pfd.dwFlags & PFD_NEED_PALETTE)
	{
		GFC::GetDebugger()->DebugOut("Creating palette\n");

		nColors = 1 << pfd.cColorBits;
		hHeap = GetProcessHeap ();

		(LPLOGPALETTE) lpPalette = (LPLOGPALETTE)HeapAlloc (hHeap, 0,
			sizeof (LOGPALETTE) + (nColors * sizeof (PALETTEENTRY)));

		lpPalette->palVersion = 0x300;
		lpPalette->palNumEntries = nColors;

		byRedMask = (1 << pfd.cRedBits) - 1;
		byGreenMask = (1 << pfd.cGreenBits) - 1;
		byBlueMask = (1 << pfd.cBlueBits) - 1;

		for (i=0; i<nColors; i++)
		{
			lpPalette->palPalEntry[i].peRed =
				(((i >> pfd.cRedShift) & byRedMask) * 255) / byRedMask;
			lpPalette->palPalEntry[i].peGreen =
				(((i >> pfd.cGreenShift) & byGreenMask) * 255) / byGreenMask;
			lpPalette->palPalEntry[i].peBlue =
				(((i >> pfd.cBlueShift) & byBlueMask) * 255) / byBlueMask;
			lpPalette->palPalEntry[i].peFlags = 0;
		}

		m_hPalette = CreatePalette (lpPalette);
		HeapFree (hHeap, 0, lpPalette);

		if (m_hPalette != NULL)
		{
			SelectPalette (hdc, m_hPalette, FALSE);
			RealizePalette (hdc);
		}
	}
}
